@@ -74,8 +74,11 @@ WXDLLIMPEXP_BASE CFURLRef wxOSXCreateURLFromFileSystemPath( const wxString& path
 #include <ApplicationServices/ApplicationServices.h>
 #endif
 
-#include "wx/bitmap.h"
+#include "wx/bmpbndl.h"
 #include "wx/window.h"
+#include "wx/toplevel.h"
+
+class wxTextProofOptions;
 
 class WXDLLIMPEXP_CORE wxMacCGContextStateSaver
 {
@@ -163,6 +166,7 @@ public :
     virtual void Check( bool check ) = 0;
     virtual void SetLabel( const wxString& text, wxAcceleratorEntry *entry ) = 0;
     virtual void Hide( bool hide = true ) = 0;
+    virtual void SetAllowsKeyEquivalentWhenHidden( bool ) {}
 
     virtual void * GetHMenuItem() = 0;
 
@@ -335,7 +339,7 @@ public :
     virtual wxInt32     GetValue() const = 0;
     virtual void        SetValue( wxInt32 v ) = 0;
     virtual wxBitmap    GetBitmap() const = 0;
-    virtual void        SetBitmap( const wxBitmap& bitmap ) = 0;
+    virtual void        SetBitmap( const wxBitmapBundle& bitmap ) = 0;
     virtual void        SetBitmapPosition( wxDirection dir ) = 0;
     virtual void        SetupTabs( const wxNotebook& WXUNUSED(notebook) ) {}
     virtual int         TabHitTest( const wxPoint & WXUNUSED(pt), long *flags ) {*flags=1; return -1;}
@@ -344,8 +348,10 @@ public :
     virtual void        Enable( bool enable ) = 0;
     virtual void        SetMinimum( wxInt32 v ) = 0;
     virtual void        SetMaximum( wxInt32 v ) = 0;
+    virtual void        SetIncrement(int value) = 0;
     virtual wxInt32     GetMinimum() const = 0;
     virtual wxInt32     GetMaximum() const = 0;
+    virtual int         GetIncrement() const = 0;
     virtual void        PulseGauge() = 0;
     virtual void        SetScrollThumb( wxInt32 value, wxInt32 thumbSize ) = 0;
 
@@ -489,7 +495,7 @@ public :
     static wxWidgetImplType*    CreateBitmapToggleButton( wxWindowMac* wxpeer,
                                     wxWindowMac* parent,
                                     wxWindowID id,
-                                    const wxBitmap& bitmap,
+                                    const wxBitmapBundle& bitmap,
                                     const wxPoint& pos,
                                     const wxSize& size,
                                     long style,
@@ -498,7 +504,7 @@ public :
     static wxWidgetImplType*    CreateBitmapButton( wxWindowMac* wxpeer,
                                     wxWindowMac* parent,
                                     wxWindowID id,
-                                    const wxBitmap& bitmap,
+                                    const wxBitmapBundle& bitmap,
                                     const wxPoint& pos,
                                     const wxSize& size,
                                     long style,
@@ -584,7 +590,7 @@ public :
     static wxWidgetImplType*    CreateStaticBitmap( wxWindowMac* wxpeer,
                                                    wxWindowMac* parent,
                                                    wxWindowID id,
-                                                   const wxBitmap& bitmap,
+                                                   const wxBitmapBundle& bitmap,
                                                    const wxPoint& pos,
                                                    const wxSize& size,
                                                    long style,
@@ -730,16 +736,22 @@ public :
     virtual void Undo() ;
     virtual bool CanRedo() const;
     virtual void Redo() ;
+    virtual void EmptyUndoBuffer() ;
     virtual int GetNumberOfLines() const ;
     virtual long XYToPosition(long x, long y) const;
     virtual bool PositionToXY(long pos, long *x, long *y) const ;
     virtual void ShowPosition(long pos) ;
     virtual int GetLineLength(long lineNo) const ;
     virtual wxString GetLineText(long lineNo) const ;
-    virtual void CheckSpelling(bool WXUNUSED(check)) { }
+#if wxUSE_SPELLCHECK
+    virtual void CheckSpelling(const wxTextProofOptions& WXUNUSED(options)) { }
+    virtual wxTextProofOptions GetCheckingOptions() const;
+#endif // wxUSE_SPELLCHECK
     virtual void EnableAutomaticQuoteSubstitution(bool WXUNUSED(enable)) {}
     virtual void EnableAutomaticDashSubstitution(bool WXUNUSED(enable)) {}
 
+    virtual void EnableNewLineReplacement(bool WXUNUSED(enable)) {}
+    virtual bool GetNewLineReplacement() { return true; }
     virtual wxSize GetBestSize() const { return wxDefaultSize; }
 
     virtual bool SetHint(const wxString& WXUNUSED(hint)) { return false; }
@@ -822,7 +834,7 @@ class wxButtonImpl
     wxButtonImpl(){}
     virtual ~wxButtonImpl(){}
 
-    virtual void SetPressedBitmap( const wxBitmap& bitmap ) = 0;
+    virtual void SetPressedBitmap( const wxBitmapBundle& bitmap ) = 0;
 } ;
 
 //
@@ -952,10 +964,13 @@ public :
 
     virtual void ShowWithoutActivating() { Show(true); }
 
-    virtual bool EnableFullScreenView(bool enable) = 0;
+    virtual bool EnableFullScreenView(bool enable, long style) = 0;
 
     virtual bool ShowFullScreen(bool show, long style)= 0;
 
+    virtual wxContentProtection GetContentProtection() const = 0;
+    virtual bool SetContentProtection(wxContentProtection contentProtection) = 0;
+
     virtual void RequestUserAttention(int flags) = 0;
 
     virtual void ScreenToWindow( int *x, int *y ) = 0;
