@@ -251,7 +251,7 @@ public:
         Returns the one and only global application object.
         Usually ::wxTheApp is used instead.
 
-        @see SetInstance()
+        @see SetInstance(), wxApp::GetGUIInstance()
     */
     static wxAppConsole* GetInstance();
 
@@ -401,6 +401,12 @@ public:
         Called from OnInit() and may be used to initialize the parser with the
         command line options for this application. The base class versions adds
         support for a few standard options only.
+
+        Note that this method should just configure @a parser to accept the
+        desired command line options by calling wxCmdLineParser::AddOption(),
+        wxCmdLineParser::AddSwitch() and similar methods, but should @e not
+        call wxCmdLineParser::Parse() as this will be done by wxWidgets itself
+        slightly later.
     */
     virtual void OnInitCmdLine(wxCmdLineParser& parser);
 
@@ -749,8 +755,7 @@ public:
             std::locale::global(std::locale(""));
         @endcode
         but be warned that locale support in C++ standard library can be poor
-        or worse under some platforms, e.g. the above line results in an
-        immediate crash under OS X up to the version 10.8.2.
+        or worse under some platforms.
 
         @since 2.9.5
      */
@@ -766,7 +771,7 @@ public:
 
         Under Windows and Linux/Unix, you should parse the command line
         arguments and check for files to be opened when starting your
-        application. Under OS X, you need to override MacOpenFiles()
+        application. Under macOS, you need to override MacOpenFiles()
         since command line arguments are used differently there.
 
         You may use the wxCmdLineParser to parse command line arguments.
@@ -784,7 +789,7 @@ public:
 
     In addition to the features provided by wxAppConsole it keeps track of
     the <em>top window</em> (see SetTopWindow()) and adds support for
-    video modes (see SetVideoMode()).
+    video modes (see SetDisplayMode()).
 
     In general, application-wide settings for GUI-only apps are accessible
     from wxApp (or from wxSystemSettings or wxSystemOptions classes).
@@ -833,6 +838,35 @@ public:
     */
     virtual wxVideoMode GetDisplayMode() const;
 
+    /**
+        Returns the current GUI wxApp object if any or @NULL otherwise.
+
+        This function should only be used in the rare cases when the same code
+        needs to work in both console and GUI applications, but needs to use
+        GUI-specific functionality if it is available, and so just calling
+        wxAppConsole::GetInstance() is insufficient while using ::wxTheApp is
+        incorrect, as the application object is not always a GUI wxApp.
+
+        For example:
+        @code
+            WXWidget handle = 0;
+            if ( wxApp* const app = wxApp::GetGUIInstance() ) {
+                if ( wxWindow* const w = app->GetTopWindow() ) {
+                    handle = w->GetHandle();
+                }
+            }
+            //else: no window to use
+
+            some_native_function_taking_a_window_handle(handle);
+        @endcode
+
+        Note that in this particular example, you could  use GetMainTopWindow()
+        which already does the same thing instead of doing it yourself.
+
+        @since 3.1.6
+    */
+    static wxAppConsole* GetGUIInstance();
+
     /**
         Returns @true if the application will exit when the top-level frame is deleted.
 
@@ -854,6 +888,18 @@ public:
     */
     bool GetUseBestVisual() const;
 
+    /**
+        Returns a pointer to the top application window if any.
+
+        This function is safe to call even before creating, or after
+        destroying, the application object, as it simply returns @NULL if it
+        doesn't exist. Otherwise it's equivalent to calling
+        @c wxTheApp->GetTopWindow().
+
+        @since 3.1.5
+     */
+    static wxWindow* GetMainTopWindow();
+
     /**
         Returns a pointer to the top window.
 
@@ -985,6 +1031,37 @@ public:
     */
     void SetUseBestVisual(bool flag, bool forceTrueColour = false);
 
+    /**
+        @name GTK-specific functions
+    */
+    //@{
+
+    /**
+        Disables the printing of various GTK messages.
+
+        This function can be called to suppress GTK diagnostic messages that
+        are output on the standard error stream by default.
+
+        The default value of the argument disables all messages, but you
+        can pass in a mask flag to specifically disable only particular
+        categories of messages.
+
+        Note that this function only works when using glib 2.50 (released in
+        September 2016) or later and does nothing with the older versions of
+        the library.
+
+        @param flags
+           The mask for the types of messages to suppress. Refer to the
+           glib documentation for the @c GLogLevelFlags enum, which defines
+           the various message types.
+
+        @onlyfor{wxgtk}
+
+        @since 3.1.6
+    */
+    static void GTKSuppressDiagnostics(int flags = -1);
+
+    //@}
 
     /**
         @name Mac-specific functions
@@ -1049,7 +1126,7 @@ public:
 
     /**
         May be overridden to indicate that the application is not a foreground
-        GUI application under OS X.
+        GUI application under macOS.
 
         This method is called during the application startup and returns @true
         by default. In this case, wxWidgets ensures that the application is ran
@@ -1063,11 +1140,26 @@ public:
         is specified in the @c Info.plist file.
 
         @onlyfor{wxosx}
-        
+
         @since 3.0.1
     */
     virtual bool OSXIsGUIApplication();
 
+    /**
+        Enable the automatic tabbing features of macOS.
+
+        This feature is native to the operating system. When it is enabled, macOS
+        will automatically place windows inside tabs and show a tab bar in the
+        application. Entries are also added to the View menu to show/hide the tab bar.
+
+        @onlyfor{wxosx}
+
+        @remarks Requires macOS 10.12+, does nothing under earlier OS versions.
+
+        @since 3.1.4
+    */
+    void OSXEnableAutomaticTabbing(bool enable);
+
     //@}
 
 };
@@ -1100,8 +1192,17 @@ public:
 #define wxDECLARE_APP( className )
 
 /**
-    This is used in the application class implementation file to make the
-    application class known to wxWidgets for dynamic construction.
+    This macro defines the application entry point and tells wxWidgets which
+    application class should be used.
+
+    The two tasks performed by this macro can be done separately by using
+    wxIMPLEMENT_APP_NO_MAIN() and wxIMPLEMENT_WXWIN_MAIN() macros, but in a
+    typical GUI application it's simpler and more convenient to use this macro
+    to do both together.
+
+    The @a className passed to this macro must be a name of the class deriving
+    from wxApp.
+
     Note that this macro requires a final semicolon.
 
     @header{wx/app.h}
@@ -1112,10 +1213,70 @@ public:
     wxIMPLEMENT_APP(MyApp);
     @endcode
 
-    @see wxDECLARE_APP()
+    @see wxDECLARE_APP(), wxIMPLEMENT_APP_CONSOLE()
 */
 #define wxIMPLEMENT_APP( className )
 
+/**
+    This macro defines the application entry point for non-GUI applications and
+    tells wxWidgets which application class should be used.
+
+    This macro is provided for symmetry with wxIMPLEMENT_APP() for the console
+    (non-GUI) applications and is equivalent to using wxIMPLEMENT_APP_NO_MAIN()
+    and wxIMPLEMENT_WXWIN_MAIN_CONSOLE().
+
+    The @a className passed to this macro must be a name of the class deriving
+    from wxApp.
+
+    Note that this macro requires a final semicolon.
+
+    @header{wx/app.h}
+
+    Example:
+
+    @code
+    wxIMPLEMENT_APP_CONSOLE(MyApp);
+    @endcode
+
+    @see wxIMPLEMENT_APP()
+*/
+#define wxIMPLEMENT_APP_CONSOLE( className )
+
+/**
+    This macro defines the application entry point appropriate for the current
+    platform.
+
+    Note that usually wxIMPLEMENT_APP() is used instead of this macro.
+
+    For most platforms, it defines @c main() function, but for GUI Windows
+    applications, it defines @c WinMain() instead.
+
+    In either case, the macro expansion includes the call to
+    wxDISABLE_DEBUG_SUPPORT() which disables debugging code in release builds.
+    If you don't use this macro, but define the entry point yourself, you
+    probably want to call wxDISABLE_DEBUG_SUPPORT() explicitly.
+
+    @header{wx/app.h}
+ */
+#define wxIMPLEMENT_WXWIN_MAIN
+
+/**
+    This macro defines the application entry point for console applications.
+
+    This macro is provided mostly for symmetry with wxIMPLEMENT_WXWIN_MAIN()
+    but is less useful, as it is also simple enough to define @c main()
+    function directly.
+
+    Please note, however, that this macro, as well as wxIMPLEMENT_APP_CONSOLE()
+    which uses it, contains the call to wxDISABLE_DEBUG_SUPPORT() which
+    disables debugging code in release builds and that if you don't use this
+    macro, but define @c main() yourself, you probably want to call
+    wxDISABLE_DEBUG_SUPPORT() from it explicitly.
+
+    @header{wx/app.h}
+ */
+#define wxIMPLEMENT_WXWIN_MAIN_CONSOLE
+
 //@}
 
 
@@ -1123,7 +1284,9 @@ public:
 /**
     The global pointer to the singleton wxApp object.
 
-    @see wxApp::GetInstance()
+    This pointer can only be used in the GUI applications.
+
+    @see wxAppConsole::GetInstance(), wxApp::GetGUIInstance()
 */
 wxApp *wxTheApp;
 
@@ -1161,7 +1324,7 @@ wxAppDerivedClass& wxGetApp();
     Notice that this function is only available if @c wxUSE_ON_FATAL_EXCEPTION
     is 1 and under Windows platform this requires a compiler with support for
     SEH (structured exception handling) which currently means only Microsoft
-    Visual C++ or a recent Borland C++ version.
+    Visual C++.
 
     @header{wx/app.h}
 */
@@ -1226,7 +1389,7 @@ bool wxSafeYield(wxWindow* win = NULL, bool onlyIfNeeded = false);
     This function initializes wxWidgets in a platform-dependent way. Use this if you
     are not using the default wxWidgets entry code (e.g. main or WinMain).
 
-    For example, you can initialize wxWidgets from an Microsoft Foundation Classes
+    For example, you can initialize wxWidgets from a Microsoft Foundation Classes
     (MFC) application using this function.
 
     @note This overload of wxEntry is available under all platforms.
