@@ -172,7 +172,7 @@ public:
     class (either wxNativePixelData for RGB bitmaps or wxAlphaPixelData
     for bitmaps with an additionally alpha channel).
 
-    Note that many wxBitmap functions take a @e type parameter, which is a 
+    Note that many wxBitmap functions take a @e type parameter, which is a
     value of the ::wxBitmapType enumeration.
     The validity of those values depends however on the platform where your program
     is running and from the wxWidgets configuration.
@@ -183,9 +183,9 @@ public:
     - wxX11 supports XPM files, XPM data, XBM data;
 
     In addition, wxBitmap can load and save all formats that wxImage can; see wxImage
-    for more info. Of course, you must have loaded the wxImage handlers 
+    for more info. Of course, you must have loaded the wxImage handlers
     (see ::wxInitAllImageHandlers() and wxImage::AddHandler).
-    Note that all available wxBitmapHandlers for a given wxWidgets port are 
+    Note that all available wxBitmapHandlers for a given wxWidgets port are
     automatically loaded at startup so you won't need to use wxBitmap::AddHandler.
 
     More on the difference between wxImage and wxBitmap: wxImage is just a
@@ -306,6 +306,20 @@ public:
     */
     wxBitmap(const wxSize& sz, int depth = wxBITMAP_SCREEN_DEPTH);
 
+    /**
+        Create a bitmap compatible with the given DC, inheriting its magnification factor
+
+        @param width
+            The width of the bitmap in pixels, must be strictly positive.
+        @param height
+            The height of the bitmap in pixels, must be strictly positive.
+        @param dc
+            DC from which the scaling factor is inherited
+
+        @since 3.1.7 (previously available only in wxMSW and wxOSX ports).
+     */
+    wxBitmap(int width, int height, const wxDC& dc);
+
     /**
         Creates a bitmap from XPM data.
 
@@ -353,15 +367,29 @@ public:
     */
     wxBitmap(const wxImage& img, int depth = wxBITMAP_SCREEN_DEPTH);
 
+    /**
+        Creates a bitmap compatible with the given DC from the given image.
+
+        This constructor initializes the bitmap with the data of the given
+        image, which must be valid, but inherits the scaling factor from the
+        given device context instead of simply using the default factor of 1.
+
+        @param img
+            Platform-independent wxImage object.
+        @param dc
+            DC from which the scaling factor is inherited
+
+        @since 3.1.7 (previously this constructor overload was only available
+            in wxMSW port)
+     */
+    wxBitmap(const wxImage& img, const wxDC& dc);
+
     /**
         Creates bitmap corresponding to the given cursor.
 
         This can be useful to display a cursor as it cannot be drawn directly
         on a window.
 
-        This constructor only exists in wxMSW and wxGTK (where it is
-        implemented for GTK+ 2.8 or later) only.
-
         @param cursor A valid wxCursor.
 
         @since 3.1.0
@@ -386,12 +414,12 @@ public:
         @param handler
             A new bitmap format handler object. There is usually only one instance
             of a given handler class in an application session.
-            
+
         Note that unlike wxImage::AddHandler, there's no documented list of
         the wxBitmapHandlers available in wxWidgets.
-        This is because they are platform-specific and most important, they are 
+        This is because they are platform-specific and most important, they are
         all automatically loaded at startup.
-        
+
         If you want to be sure that wxBitmap can load a certain type of image,
         you'd better use wxImage::AddHandler.
 
@@ -405,6 +433,15 @@ public:
     */
     static void CleanUpHandlers();
 
+    /**
+        Returns disabled (dimmed) version of the bitmap.
+
+        This method is not available when <code>wxUSE_IMAGE == 0</code>.
+
+        @since 2.9.0
+    */
+    wxBitmap ConvertToDisabled(unsigned char brightness = 255) const;
+
     /**
         Creates an image from a platform-dependent bitmap. This preserves
         mask information so that bitmaps and images can be converted back
@@ -445,7 +482,7 @@ public:
         @param height
             The height of the bitmap in pixels, must be strictly positive.
         @param dc
-            DC from wich the scaling factor is inherited
+            DC from which the scaling factor is inherited
 
         @return @true if the creation was successful.
 
@@ -454,7 +491,44 @@ public:
     bool Create(int width, int height, const wxDC& dc);
 
     /**
-        Create a bitmap with a scale factor, width and height are multiplied with that factor
+        Create a bitmap specifying its size in DPI-independent pixels and the
+        scale factor to use.
+
+        The physical size of the bitmap is obtained by multiplying the given
+        @a size by @a scale and rounding it to the closest integer.
+
+        After using this function the following postconditions are true:
+
+        - GetSize() returns @a size multiplied by @a scale
+        - GetDIPSize() returns @a size
+        - GetScaleFactor() returns @a scale
+
+        @param size
+            The size of the bitmap in DPI-independent pixels. Both width and
+            height must be strictly positive.
+        @param scale
+            Scale factor used by the bitmap, see SetScaleFactor().
+        @param depth
+            The number of bits used to represent each bitmap pixel.
+
+        @return @true if the creation was successful.
+
+        @since 3.1.6
+     */
+    bool CreateWithDIPSize(const wxSize& size,
+                           double scale,
+                           int depth = wxBITMAP_SCREEN_DEPTH);
+
+    /// @overload
+    bool CreateWithDIPSize(int width, int height,
+                           double scale,
+                           int depth = wxBITMAP_SCREEN_DEPTH);
+
+    /**
+        Create a bitmap with a scale factor.
+
+        This is an older synonym for CreateWithDIPSize(), use the new
+        function in the new code.
 
         @param width
             The width of the bitmap in pixels, must be strictly positive.
@@ -463,13 +537,13 @@ public:
         @param depth
             The number of bits used to represent each bitmap pixel.
         @param logicalScale
-            Scale factor used by the bitmap
+            Scale factor used by the bitmap, see SetScaleFactor().
 
         @return @true if the creation was successful.
 
         @since 3.1.0
     */
-    bool CreateScaled(int logwidth, int logheight, int depth, double logicalScale);
+    bool CreateScaled(int width, int height, int depth, double logicalScale);
 
     /*
         Creates a bitmap from the given data, which can be of arbitrary type.
@@ -535,6 +609,22 @@ public:
     */
     virtual int GetDepth() const;
 
+    /**
+        Returns the size of bitmap in DPI-independent units.
+
+        This assumes that the bitmap was created using the value of scale
+        factor corresponding to the current DPI (see CreateWithDIPSize()
+        and SetScaleFactor()) and returns its physical size divided by this
+        scale factor.
+
+        Unlike GetLogicalSize(), this function returns the same value under all
+        platforms and so its result should @e not be used as window or device
+        context coordinates.
+
+        @since 3.1.6
+     */
+    wxSize GetDIPSize() const;
+
     /**
         Returns the static list of bitmap format handlers.
 
@@ -543,12 +633,56 @@ public:
     static wxList& GetHandlers();
 
     /**
-        Gets the height of the bitmap in pixels.
+        Returns the height of the bitmap in physical pixels.
 
-        @see GetWidth(), GetSize()
+        @see GetWidth(), GetSize(), GetLogicalHeight()
     */
     virtual int GetHeight() const;
 
+    /**
+        Returns the height of the bitmap in logical pixels.
+
+        See GetLogicalSize() for more information.
+
+        @see GetLogicalWidth(), GetWidth()
+
+        @since 3.1.6
+     */
+    double GetLogicalHeight() const;
+
+    /**
+        Returns the size of the bitmap in logical pixels.
+
+        For the platforms using DPI-independent pixels, i.e. those where @c
+        wxHAS_DPI_INDEPENDENT_PIXELS is defined, such as wxOSX or wxGTK 3,
+        this function returns the physical size of the bitmap, as returned by
+        GetSize(), divided by its scale factor, as returned by
+        GetScaleFactor(), while for the other platforms, it simply returns the
+        same thing as GetSize().
+
+        This ensures that the result of this function is always expressed in
+        the pixel coordinates appropriate for the current platform, i.e. its
+        return value is always in logical pixels, used for window and wxDC
+        coordinates, whether these pixels are the same as physical pixels,
+        which are returned by GetSize(), or not.
+
+        @see GetLogicalWidth(), GetLogicalHeight(), GetSize()
+
+        @since 2.9.5
+     */
+    wxSize GetLogicalSize() const;
+
+    /**
+        Returns the width of the bitmap in logical pixels.
+
+        See GetLogicalSize() for more information.
+
+        @see GetLogicalHeight(), GetWidth()
+
+        @since 3.1.6
+     */
+    double GetLogicalWidth() const;
+
     /**
         Gets the associated mask (if any) which may have been loaded from a file
         or set for the bitmap.
@@ -572,27 +706,70 @@ public:
     virtual wxBitmap GetSubBitmap(const wxRect& rect) const;
 
     /**
-        Returns the size of the bitmap in pixels.
+        Returns the scale factor of this bitmap.
+
+        Scale factor is 1 by default, but can be greater to indicate that the
+        size of bitmap in logical, DPI-independent pixels is smaller than its
+        actual size in physical pixels. Bitmaps with scale factor greater than
+        1 must be used in high DPI to appear sharp on the screen.
+
+        Note that the scale factor is only used in the ports where logical
+        pixels are not the same as physical ones, such as wxOSX or wxGTK3, and
+        this function always returns 1 under the other platforms.
+
+        @see SetScaleFactor(), GetLogicalWidth(), GetLogicalHeight(), GetLogicalSize()
+
+        @since 2.9.5
+     */
+    virtual double GetScaleFactor() const;
+
+    /**
+        Returns the height of the bitmap in logical pixels.
+
+        This is an older synonym for GetLogicalHeight(), use the new function
+        in the new code.
+
+        @since 2.9.5
+     */
+    double GetScaledHeight() const;
+
+    /**
+        Returns the size of the bitmap in logical pixels.
+
+        This is an older synonym for GetLogicalSize(), use the new function in
+        the new code.
+
+        @since 2.9.5
+     */
+    wxSize GetScaledSize() const;
+
+    /**
+        Returns the width of the bitmap in logical pixels.
+
+        This is an older synonym for GetLogicalWidth(), use the new function in
+        the new code.
+
+        @since 2.9.5
+     */
+    double GetScaledWidth() const;
+
+    /**
+        Returns the size of the bitmap in physical pixels.
+
+        The return value of this function doesn't depend on the scale factor,
+        it is always the physical size of the bitmap, i.e. corresponding to the
+        actual number of pixels in it.
 
         @since 2.9.0
 
-        @see GetHeight(), GetWidth()
+        @see GetHeight(), GetWidth(), GetLogicalSize()
     */
     wxSize GetSize() const;
 
     /**
-        Returns disabled (dimmed) version of the bitmap.
+        Returns the width of the bitmap in physical pixels.
 
-        This method is not available when <code>wxUSE_IMAGE == 0</code>.
-
-        @since 2.9.0
-    */
-    wxBitmap ConvertToDisabled(unsigned char brightness = 255) const;
-
-    /**
-        Gets the width of the bitmap in pixels.
-
-        @see GetHeight(), GetSize()
+        @see GetHeight(), GetSize(), GetLogicalWidth()
     */
     virtual int GetWidth() const;
 
@@ -665,7 +842,7 @@ public:
         @endcode
         in your application startup code.
 
-        However under OS X this function uses native image loading and so
+        However under macOS this function uses native image loading and so
         doesn't require wxWidgets PNG support.
 
         @since 2.9.5
@@ -685,6 +862,21 @@ public:
     */
     static bool RemoveHandler(const wxString& name);
 
+    /**
+        Rescale the given bitmap to the requested size.
+
+        This function is just a convenient wrapper for wxImage::Rescale() used
+        to resize the given @a bmp to the requested size. If you need more
+        control over resizing, e.g. to specify the quality option different
+        from ::wxIMAGE_QUALITY_NEAREST used by this function, please use the
+        wxImage function directly instead.
+
+        Both the bitmap itself and size must be valid.
+
+        @since 3.1.6
+     */
+    static void Rescale(wxBitmap& bmp, const wxSize& sizeNeeded);
+
     /**
         Saves a bitmap in the named file.
 
@@ -707,17 +899,21 @@ public:
                           const wxPalette* palette = NULL) const;
 
     /**
-        Sets the depth member (does not affect the bitmap data).
+         @deprecated This function is deprecated since version 3.1.2, dimensions
+            and depth can only be set at construction time.
 
-        @todo since these functions do not affect the bitmap data,
-              why they exist??
+        Sets the depth member (does not affect the bitmap data).
 
         @param depth
             Bitmap depth.
+
     */
     virtual void SetDepth(int depth);
 
     /**
+        @deprecated This function is deprecated since version 3.1.2, dimensions
+            and depth can only be set at construction time.
+
         Sets the height member (does not affect the bitmap data).
 
         @param height
@@ -725,11 +921,32 @@ public:
     */
     virtual void SetHeight(int height);
 
+    /**
+        Sets the bitmap scale factor.
+
+        This doesn't change the bitmap actual size or its contents, but changes
+        its scale factor, so that it appears in a smaller size when it is drawn
+        on screen: e.g. setting @a scale to 2 means that the bitmap will be
+        twice smaller (in each direction) when drawn on screen in the ports in
+        which logical and physical pixels differ (i.e. wxOSX and wxGTK3, but
+        not wxMSW).
+
+        When creating a new bitmap, CreateWithDIPSize() can be used to
+        specify the correct scale factor from the beginning.
+
+        @since 3.1.6
+     */
+    virtual void SetScaleFactor(double scale);
+
     /**
         Sets the mask for this bitmap.
 
         @remarks The bitmap object owns the mask once this has been called.
 
+        @note A mask can be set also for bitmap with an alpha channel but
+        doing so under wxMSW is not recommended because performance of drawing
+        such bitmap is not very good.
+
         @see GetMask(), wxMask
     */
     virtual void SetMask(wxMask* mask);
@@ -745,6 +962,9 @@ public:
     virtual void SetPalette(const wxPalette& palette);
 
     /**
+        @deprecated This function is deprecated since version 3.1.2, dimensions
+            and depth can only be set at construction time.
+
         Sets the width member (does not affect the bitmap data).
 
         @param width
@@ -770,6 +990,10 @@ wxBitmap wxNullBitmap;
     When associated with a bitmap and drawn in a device context, the unmasked
     area of the bitmap will be drawn, and the masked area will not be drawn.
 
+    @note A mask can be associated also with a bitmap with an alpha channel
+    but drawing such bitmaps under wxMSW may be slow so using them should be
+    avoided if drawing performance is an important factor.
+
     @library{wxcore}
     @category{gdi}
 
