@@ -1156,6 +1156,56 @@ public:
     void DisableOffset();
     bool OffsetEnabled() const;
 
+    /**
+        Convert DPI-independent pixel values to the value in pixels appropriate
+        for the graphics context.
+
+        See wxWindow::FromDIP(const wxSize& sz) and wxDC::FromDIP(const wxSize& sz)
+        for more info about converting device independent pixel values.
+
+        @since 3.1.7
+     */
+    wxSize FromDIP(const wxSize& sz) const;
+
+    /// @overload
+    wxPoint FromDIP(const wxPoint& pt) const;
+
+    /**
+        Convert DPI-independent value in pixels to the value in pixels
+        appropriate for the graphics context.
+
+        This is the same as FromDIP(const wxSize& sz) overload, but assumes
+        that the resolution is the same in horizontal and vertical directions.
+
+        @since 3.1.7
+     */
+    int FromDIP(int d) const;
+
+    /**
+        Convert pixel values of the current graphics context to DPI-independent
+        pixel values.
+
+        See wxWindow::ToDIP(const wxSize& sz) and wxDC::ToDIP(const wxSize& sz)
+        for more info about converting device independent pixel values.
+
+        @since 3.1.7
+     */
+    wxSize ToDIP(const wxSize& sz) const;
+
+    /// @overload
+    wxPoint ToDIP(const wxPoint& pt) const;
+
+    /**
+        Convert pixel values of the current graphics context to DPI-independent
+        pixel values.
+
+        This is the same as ToDIP(const wxSize& sz) overload, but assumes
+        that the resolution is the same in horizontal and vertical directions.
+
+        @since 3.1.7
+     */
+    int ToDIP(int d) const;
+
     /** @}
     */
 };
