@@ -250,7 +250,7 @@ public:
 
         @since 3.1.5
     */
-    virtual wxVersionInfo GetVersionInfo(const wxString& backend);
+    virtual wxVersionInfo GetVersionInfo();
 };
 
 /**
@@ -307,9 +307,15 @@ public:
     although currently just one is available. It differs from wxHtmlWindow in
     that each backend is actually a full rendering engine, Internet Explorer or Edge on MSW and
     WebKit on macOS and GTK. This allows the correct viewing of complex pages with
-    javascript and css.
+    JavaScript and CSS.
 
-    @section descriptions Backend Descriptions
+    @section backend_descriptions Backend Descriptions
+
+    This class supports using multiple backends, corresponding to different
+    implementations of the same functionality. Under macOS and Unix platforms
+    only a single, WebKit-based, backend is currently provided, but under MSW
+    both the legacy IE backend and the new Edge backend exist. Backends are
+    identified by their names, documented in the backend descriptions below.
 
     @subsection wxWEBVIEW_BACKEND_IE wxWEBVIEW_BACKEND_IE (MSW)
 
@@ -323,6 +329,9 @@ public:
 
     This backend has full support for custom schemes and virtual file systems.
 
+    The predefined @c wxWebViewBackendIE constant contains the name of this
+    backend and can be used to explicitly select it when using wxWebView::New().
+
     @note If you plan to display any modern web content you should consider using @ref wxWEBVIEW_BACKEND_EDGE,
           as Internet Explorer is not supported anymore by Microsoft.
 
@@ -339,7 +348,7 @@ public:
     - With CMake just enable @c wxUSE_WEBVIEW_EDGE
     - When not using CMake:
         - Download the <a href="https://aka.ms/webviewnuget">WebView2 SDK</a>
-        nuget package (Version 1.0.622.22 or newer)
+        nuget package (Version 1.0.705.50 or newer)
         - Extract the package (it's a zip archive) to @c wxWidgets/3rdparty/webview2
         (you should have @c 3rdparty/webview2/build/native/include/WebView2.h
         file after unpacking it)
@@ -353,9 +362,12 @@ public:
       loaded and Edge (Chromium) is installed)
     - Make sure to add a note about using the WebView2 SDK to your application
       documentation, as required by its licence
+    - With Visual Studio 2019 or newer @c wxUSE_WEBVIEW_EDGE_STATIC can be used
+      to static link the loader and remove the dependency on @c WebView2Loader.dll
+      at runtime.
 
     If enabled and available at runtime Edge will be selected as the default
-    backend. If you require the IE backend use @c wxWEBVIEW_BACKEND_IE when
+    backend. If you require the IE backend use @c wxWebViewBackendIE when
     using wxWebView::New().
 
     If your application should use a
@@ -364,6 +376,9 @@ public:
     wxWebViewEdge::MSWSetBrowserExecutableDir() to specify its usage before
     using the Edge backend.
 
+    The predefined @c wxWebViewBackendEdge constant contains the name of this
+    backend.
+
     @subsection wxWEBVIEW_WEBKIT wxWEBVIEW_WEBKIT (GTK)
 
     Under GTK the WebKit backend uses
@@ -376,6 +391,9 @@ public:
     resources such as images and stylesheets are currently loaded using the
     data:// scheme.
 
+    The predefined @c wxWebViewBackendWebKit constant contains the name of this
+    backend.
+
     @subsection wxWEBVIEW_WEBKIT2 wxWEBVIEW_WEBKIT2 (GTK3)
 
     Under GTK3 the WebKit2 version of <a href="http://webkitgtk.org/">WebKitGTK+</a>
@@ -385,6 +403,9 @@ public:
     All features are
     supported except for clearing and enabling / disabling the history.
 
+    The predefined @c wxWebViewBackendWebKit constant contains the name of this
+    backend.
+
     @subsection wxWEBVIEW_WEBKIT_MACOS wxWEBVIEW_WEBKIT (macOS)
 
     The macOS WebKit backend uses Apple's
@@ -401,6 +422,9 @@ public:
     For further details see the documentation on NSAppTransportSecurity
     <a target=_new href="https://developer.apple.com/documentation/bundleresources/information_property_list/nsapptransportsecurity">here</a>
 
+    The predefined @c wxWebViewBackendWebKit constant contains the name of this
+    backend.
+
     @section async Asynchronous Notifications
 
     Many of the methods in wxWebView are asynchronous, i.e. they return
@@ -464,6 +488,10 @@ public:
         Process a @c wxEVT_WEBVIEW_SCRIPT_MESSAGE_RECEIVED event
         only available in wxWidgets 3.1.5 or later. For usage details see
         AddScriptMessageHandler().
+    @event{wxEVT_WEBVIEW_SCRIPT_RESULT(id, func)}
+        Process a @c wxEVT_WEBVIEW_SCRIPT_RESULT event
+        only available in wxWidgets 3.1.6 or later. For usage details see
+        RunScriptAsync().
     @endEventTable
 
     @since 2.9.3
@@ -537,6 +565,14 @@ public:
     /**
         Allows to check if a specific backend is currently available.
 
+        For example, to check for Edge backend availability:
+        @code
+        if ( wxWebView::IsBackendAvailable(wxWebViewBackendEdge) )
+        {
+            ... enable some extra functionality not available with the IE backend ...
+        }
+        @endcode
+
         @since 3.1.4
     */
     static bool IsBackendAvailable(const wxString& backend);
@@ -681,6 +717,11 @@ public:
     /**
         Runs the given JavaScript code.
 
+        @note Because of various potential issues it's recommended to use
+            RunScriptAsync() instead of this method. This is especially true
+            if you plan to run code from a webview event and will also prevent
+            unintended side effects on the UI outside of the webview.
+
         JavaScript code is executed inside the browser control and has full
         access to DOM and other browser-provided functionality. For example,
         this code
@@ -735,15 +776,38 @@ public:
             @NULL if it is not needed. This parameter is new since wxWidgets
             version 3.1.1.
         @return @true if there is a result, @false if there is an error.
+
+        @see RunScriptAsync()
     */
     virtual bool RunScript(const wxString& javascript, wxString* output = NULL) const = 0;
 
+    /**
+        Runs the given JavaScript code asynchronously and returns the result
+        via a @c wxEVT_WEBVIEW_SCRIPT_RESULT.
+
+        The script result value can be retrieved via wxWebViewEvent::GetString().
+        If the execution fails wxWebViewEvent::IsError() will return @true. In this
+        case additional script execution error information maybe available
+        via wxWebViewEvent::GetString().
+
+        @param javascript JavaScript code to execute.
+        @param clientData Arbirary pointer to data that can be retrieved from
+            the result event.
+
+        @note The IE backend does not support async script execution.
+
+        @since 3.1.6
+        @see RunScript()
+    */
+    virtual void RunScriptAsync(const wxString& javascript, void* clientData = NULL) const;
+
+
     /**
         Add a script message handler with the given name.
 
         To use the script message handler from javascript use
-        @c window.<name>.postMessage(<messageBody>) where <name> corresponds the value
-        of the name parameter. The <messageBody> will be available to the application
+        @c `window.<name>.postMessage(<messageBody>)` where `<name>` corresponds the value
+        of the name parameter. The `<messageBody>` will be available to the application
         via a @c wxEVT_WEBVIEW_SCRIPT_MESSAGE_RECEIVED event.
 
         Sample C++ code receiving a script message:
@@ -1250,6 +1314,10 @@ public:
         Process a @c wxEVT_WEBVIEW_SCRIPT_MESSAGE_RECEIVED event
         only available in wxWidgets 3.1.5 or later. For usage details see
         wxWebView::AddScriptMessageHandler().
+    @event{wxEVT_WEBVIEW_SCRIPT_RESULT(id, func)}
+        Process a @c wxEVT_WEBVIEW_SCRIPT_RESULT event
+        only available in wxWidgets 3.1.6 or later. For usage details see
+        wxWebView::RunScriptAsync().
     @endEventTable
 
     @since 2.9.3
@@ -1294,6 +1362,14 @@ public:
         @since 3.1.5
     */
     const wxString& GetMessageHandler() const;
+
+    /**
+        Returns true the script execution failed. Only valid for events of type
+        @c wxEVT_WEBVIEW_SCRIPT_RESULT
+
+        @since 3.1.6
+    */
+    bool IsError() const;
 };
 
 
@@ -1303,3 +1379,6 @@ wxEventType wxEVT_WEBVIEW_LOADED;
 wxEventType wxEVT_WEBVIEW_ERROR;
 wxEventType wxEVT_WEBVIEW_NEWWINDOW;
 wxEventType wxEVT_WEBVIEW_TITLE_CHANGED;
+wxEventType wxEVT_WEBVIEW_FULLSCREEN_CHANGED;
+wxEventType wxEVT_WEBVIEW_SCRIPT_MESSAGE_RECEIVED;
+wxEventType wxEVT_WEBVIEW_SCRIPT_RESULT;
