@@ -12,25 +12,15 @@
 // For compilers that support precompilation, includes "wx/wx.h".
 #include "wx/wxprec.h"
 
-#ifdef __BORLANDC__
-#pragma hdrstop
-#endif
 
 #ifndef WX_PRECOMP
     #include "wx/wx.h"
 #endif
 
+#include "wx/artprov.h"
 #include "wx/dataview.h"
 #include "mymodels.h"
 
-// ----------------------------------------------------------------------------
-// resources
-// ----------------------------------------------------------------------------
-
-#include "null.xpm"
-#include "wx_small.xpm"
-
-
 // ----------------------------------------------------------------------------
 // MyMusicTreeModel
 // ----------------------------------------------------------------------------
@@ -43,6 +33,8 @@ MyMusicTreeModel::MyMusicTreeModel()
     m_pop = new MyMusicTreeModelNode( m_root, "Pop music" );
     m_pop->Append(
         new MyMusicTreeModelNode( m_pop, "You are not alone", "Michael Jackson", 1995 ) );
+    m_pop->Append(
+        new MyMusicTreeModelNode( m_pop, "Yesterday", "The Beatles", -1 /* not specified */ ) );
     m_pop->Append(
         new MyMusicTreeModelNode( m_pop, "Take a bow", "Madonna", 1994 ) );
     m_root->Append( m_pop );
@@ -153,6 +145,21 @@ void MyMusicTreeModel::Delete( const wxDataViewItem &item )
     // notify control
     ItemDeleted( parent, item );
 }
+void MyMusicTreeModel::Clear()
+{
+    m_pop       = NULL;
+    m_classical = NULL;
+    m_ninth     = NULL;
+
+    while (!m_root->GetChildren().IsEmpty())
+    {
+        MyMusicTreeModelNode* node = m_root->GetNthChild(0);
+        m_root->GetChildren().Remove(node);
+        delete node;
+    }
+
+    Cleared();
+}
 
 int MyMusicTreeModel::Compare( const wxDataViewItem &item1, const wxDataViewItem &item2,
                                unsigned int column, bool ascending ) const
@@ -196,7 +203,8 @@ void MyMusicTreeModel::GetValue( wxVariant &variant,
         variant = node->m_artist;
         break;
     case 2:
-        variant = (long) node->m_year;
+        if (node->m_year != -1)
+            variant = (long) node->m_year;
         break;
     case 3:
         variant = node->m_quality;
@@ -205,7 +213,9 @@ void MyMusicTreeModel::GetValue( wxVariant &variant,
         variant = 80L;  // all music is very 80% popular
         break;
     case 5:
-        if (GetYear(item) < 1900)
+        if (node->m_year == -1)
+            variant = "n/a";
+        else if (node->m_year < 1900)
             variant = "old";
         else
             variant = "new";
@@ -251,7 +261,16 @@ bool MyMusicTreeModel::IsEnabled( const wxDataViewItem &item,
     MyMusicTreeModelNode *node = (MyMusicTreeModelNode*) item.GetID();
 
     // disable Beethoven's ratings, his pieces can only be good
-    return !(col == 3 && node->m_artist.EndsWith("Beethoven"));
+    if (col == 3 && node->m_artist.EndsWith("Beethoven"))
+        return false;
+
+    // also disable editing the year when it's not specified, this doesn't work
+    // because the editor needs some initial value
+    if (col == 2 && node->m_year == -1)
+        return false;
+
+    // otherwise allow editing
+    return true;
 }
 
 wxDataViewItem MyMusicTreeModel::GetParent( const wxDataViewItem &item ) const
@@ -271,7 +290,7 @@ wxDataViewItem MyMusicTreeModel::GetParent( const wxDataViewItem &item ) const
 
 bool MyMusicTreeModel::IsContainer( const wxDataViewItem &item ) const
 {
-    // the invisble root node can have children
+    // the invisible root node can have children
     // (in our model always "MyMusic")
     if (!item.IsOk())
         return true;
@@ -292,7 +311,7 @@ unsigned int MyMusicTreeModel::GetChildren( const wxDataViewItem &parent,
 
     if (node == m_classical)
     {
-        MyMusicTreeModel *model = (MyMusicTreeModel*)(const MyMusicTreeModel*) this;
+        MyMusicTreeModel* model = const_cast<MyMusicTreeModel*>(this);
         model->m_classicalMusicIsKnownToControl = true;
     }
 
@@ -312,6 +331,19 @@ unsigned int MyMusicTreeModel::GetChildren( const wxDataViewItem &parent,
 }
 
 
+// ----------------------------------------------------------------------------
+// MyLongMusicTreeModel
+// ----------------------------------------------------------------------------
+
+MyLongMusicTreeModel::MyLongMusicTreeModel() : MyMusicTreeModel()
+{
+    for (int i = 0; i < 50; i++)
+    {
+        AddToClassical("The Four Seasons", "Antonio Vivaldi", 1721);
+        AddToClassical("La costanza trionfante degl'amori e de gl'odii", "Antonio Vivaldi", 1716);
+    }
+}
+
 
 // ----------------------------------------------------------------------------
 // MyListModel
@@ -329,9 +361,12 @@ static int my_sort( int *v1, int *v2 )
 
 #define INITIAL_NUMBER_OF_ITEMS 10000
 
-MyListModel::MyListModel() :
+MyListModel::MyListModel(int modelFlags) :
         wxDataViewVirtualListModel( INITIAL_NUMBER_OF_ITEMS )
 {
+    const wxString multiLineText = L"top (\u1ED6)\ncentre\nbottom (g)";
+    const bool useMultiLine = (modelFlags & MODEL_USE_MULTI_LINE_TEXT) != 0;
+
     // the first 100 items are really stored in this model;
     // all the others are synthesized on request
     static const unsigned NUMBER_REAL_ITEMS = 100;
@@ -339,17 +374,22 @@ MyListModel::MyListModel() :
     m_toggleColValues.reserve(NUMBER_REAL_ITEMS);
     m_textColValues.reserve(NUMBER_REAL_ITEMS);
     m_toggleColValues.push_back(false);
-    m_textColValues.push_back("first row with long label to test ellipsization");
+    m_textColValues.push_back(useMultiLine
+        ? multiLineText
+        : wxString("first row with long label to test ellipsization"));
     for (unsigned int i = 1; i < NUMBER_REAL_ITEMS; i++)
     {
         m_toggleColValues.push_back(false);
         m_textColValues.push_back(wxString::Format("real row %d", i));
     }
 
-    m_iconColValues.assign(NUMBER_REAL_ITEMS, "test");
+    m_iconColValues.assign(NUMBER_REAL_ITEMS,
+        useMultiLine ? multiLineText : wxString("test"));
 
-    m_icon[0] = wxIcon( null_xpm );
-    m_icon[1] = wxIcon( wx_small_xpm );
+    const wxSize size = GetIconSizeFromModelFlags(modelFlags);
+
+    m_icon[0] = wxArtProvider::GetBitmapBundle(wxART_QUESTION, wxART_LIST, size);
+    m_icon[1] = wxArtProvider::GetBitmapBundle(wxART_WX_LOGO, wxART_LIST, size);
 }
 
 void MyListModel::Prepend( const wxString &text )
@@ -399,7 +439,7 @@ void MyListModel::DeleteItems( const wxDataViewItemArray &items )
 
     // Sort in descending order so that the last
     // row will be deleted first. Otherwise the
-    // remaining indeces would all be wrong.
+    // remaining indices would all be wrong.
     rows.Sort( my_sort_reverse );
     for (i = 0; i < rows.GetCount(); i++)
     {
@@ -424,13 +464,6 @@ void MyListModel::GetValueByRow( wxVariant &variant,
 {
     switch ( col )
     {
-        case Col_Toggle:
-            if (row >= m_toggleColValues.size())
-                variant = false;
-            else
-                variant = m_toggleColValues[row];
-            break;
-
         case Col_EditableText:
             if (row >= m_textColValues.GetCount())
                 variant = wxString::Format( "virtual row %d", row );
@@ -438,15 +471,22 @@ void MyListModel::GetValueByRow( wxVariant &variant,
                 variant = m_textColValues[ row ];
             break;
 
-        case Col_IconText:
+        case Col_ToggleIconText:
             {
                 wxString text;
+                wxCheckBoxState state;
                 if ( row >= m_iconColValues.GetCount() )
+                {
                     text = "virtual icon";
+                    state = wxCHK_UNDETERMINED;
+                }
                 else
+                {
                     text = m_iconColValues[row];
+                    state = m_toggleColValues[row] ? wxCHK_CHECKED : wxCHK_UNCHECKED;
+                }
 
-                variant << wxDataViewIconText(text, m_icon[row % 2]);
+                variant << wxDataViewCheckIconText(text, m_icon[row % 2], state);
             }
             break;
 
@@ -492,9 +532,6 @@ bool MyListModel::GetAttrByRow( unsigned int row, unsigned int col,
 {
     switch ( col )
     {
-        case Col_Toggle:
-            return false;
-
         case Col_EditableText:
         case Col_Date:
             if (row < m_toggleColValues.size())
@@ -508,7 +545,7 @@ bool MyListModel::GetAttrByRow( unsigned int row, unsigned int col,
             }
             return false;
 
-        case Col_IconText:
+        case Col_ToggleIconText:
             if ( !(row % 2) )
                 return false;
             attr.SetColour(*wxYELLOW);
@@ -565,20 +602,13 @@ bool MyListModel::SetValueByRow( const wxVariant &variant,
 {
     switch ( col )
     {
-        case Col_Toggle:
-            if (row >= m_toggleColValues.size())
-                return false;
-
-            m_toggleColValues[row] = variant.GetBool();
-            return true;
-
         case Col_EditableText:
-        case Col_IconText:
+        case Col_ToggleIconText:
             if (row >= m_textColValues.GetCount())
             {
                 // the item is not in the range of the items
                 // which we store... for simplicity, don't allow editing it
-                wxLogError( "Cannot edit rows with an index greater than %d",
+                wxLogError( "Cannot edit rows with an index greater than %zu",
                             m_textColValues.GetCount() );
                 return false;
             }
@@ -587,11 +617,13 @@ bool MyListModel::SetValueByRow( const wxVariant &variant,
             {
                 m_textColValues[row] = variant.GetString();
             }
-            else // col == Col_IconText
+            else // col == Col_ToggleIconText
             {
-                wxDataViewIconText iconText;
-                iconText << variant;
-                m_iconColValues[row] = iconText.GetText();
+                wxDataViewCheckIconText checkIconText;
+                checkIconText << variant;
+                m_toggleColValues[row] =
+                    checkIconText.GetCheckedState() == wxCHK_CHECKED;
+                m_iconColValues[row] = checkIconText.GetText();
             }
             return true;
 
@@ -621,3 +653,15 @@ bool MyListStoreDerivedModel::IsEnabledByRow(unsigned int row, unsigned int col)
     // disabled the last two checkboxes
     return !(col == 0 && 8 <= row && row <= 9);
 }
+
+// ----------------------------------------------------------------------------
+// MyListStoreHasValueModel
+// ----------------------------------------------------------------------------
+
+bool MyListStoreHasValueModel::HasValue(const wxDataViewItem &item, unsigned int col) const
+{
+    unsigned int row = GetRow( item );
+    // the diagonal entries don't have values. This is just a silly example to demonstrate the
+    // usage of overriding HasValue to specify that some columns don't have values for some items
+    return row != col;
+}
