@@ -18,9 +18,6 @@
 
 #include "wx/wxprec.h"
 
-#ifdef __BORLANDC__
-    #pragma hdrstop
-#endif
 
 #if wxUSE_AUI
 
@@ -30,6 +27,7 @@
 #include "wx/aui/tabart.h"
 
 #ifndef WX_PRECOMP
+    #include "wx/app.h"
     #include "wx/settings.h"
     #include "wx/dcclient.h"
     #include "wx/image.h"
@@ -42,12 +40,12 @@
 #endif
 
 #ifdef __WXGTK__
-#include <gtk/gtk.h>
 #include "wx/renderer.h"
 #ifdef __WXGTK20__
-   #include "wx/gtk/private/gtk2-compat.h"
+    #include "wx/gtk/private/wrapgtk.h"
 #else
-   #define gtk_widget_is_drawable GTK_WIDGET_DRAWABLE
+    #include <gtk/gtk.h>
+    #define gtk_widget_is_drawable GTK_WIDGET_DRAWABLE
 #endif
 #ifdef __WXGTK3__
     #include "wx/graphics.h"
@@ -55,15 +53,8 @@
 #endif
 #endif
 
+#include <math.h>
 
-// -- wxAuiDefaultDockArt class implementation --
-
-// wxAuiDefaultDockArt is an art provider class which does all of the drawing for
-// wxAuiManager.  This allows the library caller to customize the dock art
-// (probably by deriving from this class), or to completely replace all drawing
-// with custom dock art (probably by writing a new stand-alone class derived
-// from the wxAuiDockArt base class). The active dock art class can be set via
-// wxAuiManager::SetDockArt()
 wxColor wxAuiLightContrastColour(const wxColour& c)
 {
     int amount = 120;
@@ -76,19 +67,59 @@ wxColor wxAuiLightContrastColour(const wxColour& c)
     return c.ChangeLightness(amount);
 }
 
+inline float wxAuiGetSRGB(float r) {
+    return r <= 0.03928f ? r / 12.92f : std::pow((r + 0.055f) / 1.055f, 2.4f);
+}
+
+float wxAuiGetRelativeLuminance(const wxColour& c)
+{
+    // based on https://www.w3.org/TR/2008/REC-WCAG20-20081211/#relativeluminancedef
+    return
+        0.2126f * wxAuiGetSRGB(c.Red()   / 255.0f) +
+        0.7152f * wxAuiGetSRGB(c.Green() / 255.0f) +
+        0.0722f * wxAuiGetSRGB(c.Blue()  / 255.0f);
+}
+
+float wxAuiGetColourContrast(const wxColour& c1, const wxColour& c2)
+{
+    // based on https://www.w3.org/TR/UNDERSTANDING-WCAG20/visual-audio-contrast7.html
+    float L1 = wxAuiGetRelativeLuminance(c1);
+    float L2 = wxAuiGetRelativeLuminance(c2);
+    return L1 > L2 ? (L1 + 0.05f) / (L2 + 0.05f) : (L2 + 0.05f) / (L1 + 0.05f);
+}
+
 // wxAuiBitmapFromBits() is a utility function that creates a
 // masked bitmap from raw bits (XBM format)
 wxBitmap wxAuiBitmapFromBits(const unsigned char bits[], int w, int h,
                              const wxColour& color)
 {
     wxImage img = wxBitmap((const char*)bits, w, h).ConvertToImage();
-    img.Replace(0,0,0,123,123,123);
-    img.Replace(255,255,255,color.Red(),color.Green(),color.Blue());
-    img.SetMaskColour(123,123,123);
+    img.InitAlpha();
+    const int newr = color.Red();
+    const int newg = color.Green();
+    const int newb = color.Blue();
+    const int newa = color.Alpha();
+    for (int x = 0; x < w; x++)
+    {
+        for (int y = 0; y < h; y++)
+        {
+            int r = img.GetRed(x, y);
+            int g = img.GetGreen(x, y);
+            int b = img.GetBlue(x, y);
+            if (r == 0 && g == 0 && b == 0)
+            {
+                img.SetAlpha(x, y, wxALPHA_TRANSPARENT);
+            }
+            else
+            {
+                img.SetRGB(x, y, newr, newg, newb);
+                img.SetAlpha(x, y, newa);
+            }
+        }
+    }
     return wxBitmap(img);
 }
 
-
 static void DrawGradientRectangle(wxDC& dc,
                                   const wxRect& rect,
                                   const wxColour& start_color,
@@ -154,45 +185,17 @@ wxString wxAuiChopText(wxDC& dc, const wxString& text, int max_size)
     return ret;
 }
 
+// -- wxAuiDefaultDockArt class implementation --
+
+// wxAuiDefaultDockArt is an art provider class which does all of the drawing for
+// wxAuiManager.  This allows the library caller to customize the dock art
+// (probably by deriving from this class), or to completely replace all drawing
+// with custom dock art (probably by writing a new stand-alone class derived
+// from the wxAuiDockArt base class). The active dock art class can be set via
+// wxAuiManager::SetDockArt()
 wxAuiDefaultDockArt::wxAuiDefaultDockArt()
 {
-#if defined( __WXMAC__ ) && wxOSX_USE_COCOA_OR_CARBON
-    wxColor baseColour = wxColour( wxMacCreateCGColorFromHITheme(kThemeBrushToolbarBackground));
-#else
-    wxColor baseColour = wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE);
-#endif
-
-    // the baseColour is too pale to use as our base colour,
-    // so darken it a bit --
-    if ((255-baseColour.Red()) +
-        (255-baseColour.Green()) +
-        (255-baseColour.Blue()) < 60)
-    {
-        baseColour = baseColour.ChangeLightness(92);
-    }
-
-    m_baseColour = baseColour;
-    wxColor darker1Colour = baseColour.ChangeLightness(85);
-    wxColor darker2Colour = baseColour.ChangeLightness(75);
-    wxColor darker3Colour = baseColour.ChangeLightness(60);
-    //wxColor darker4Colour = baseColour.ChangeLightness(50);
-    wxColor darker5Colour = baseColour.ChangeLightness(40);
-
-    m_activeCaptionColour = wxSystemSettings::GetColour(wxSYS_COLOUR_HIGHLIGHT);
-    m_activeCaptionGradientColour = wxAuiLightContrastColour(wxSystemSettings::GetColour(wxSYS_COLOUR_HIGHLIGHT));
-    m_activeCaptionTextColour = wxSystemSettings::GetColour(wxSYS_COLOUR_HIGHLIGHTTEXT);
-    m_inactiveCaptionColour = darker1Colour;
-    m_inactiveCaptionGradientColour = baseColour.ChangeLightness(97);
-    m_inactiveCaptionTextColour = *wxBLACK;
-
-    m_sashBrush = wxBrush(baseColour);
-    m_backgroundBrush = wxBrush(baseColour);
-    m_gripperBrush = wxBrush(baseColour);
-
-    m_borderPen = wxPen(darker2Colour);
-    m_gripperPen1 = wxPen(darker5Colour);
-    m_gripperPen2 = wxPen(darker3Colour);
-    m_gripperPen3 = *wxWHITE_PEN;
+    UpdateColoursFromSystem();
 
 #ifdef __WXMAC__
     m_captionFont = *wxSMALL_FONT;
@@ -204,25 +207,31 @@ wxAuiDefaultDockArt::wxAuiDefaultDockArt()
 #if defined( __WXMAC__ ) && wxOSX_USE_COCOA_OR_CARBON
     SInt32 height;
     GetThemeMetric( kThemeMetricSmallPaneSplitterHeight , &height );
-    m_sashSize = height;
+    m_sashSize     = height;
 #elif defined(__WXGTK__)
-    m_sashSize = wxRendererNative::Get().GetSplitterParams(NULL).widthSash;
+    m_sashSize     = wxRendererNative::Get().GetSplitterParams(NULL).widthSash;
 #else
-    m_sashSize = 4;
+    m_sashSize     = wxWindow::FromDIP( 4, NULL);
 #endif
-    m_captionSize = 17;
-    m_borderSize = 1;
-    m_buttonSize = 14;
-    m_gripperSize = 9;
+    m_captionSize  = wxWindow::FromDIP(17, NULL);
+    m_borderSize   = 1;
+    m_buttonSize   = wxWindow::FromDIP(14, NULL);
+    m_gripperSize  = wxWindow::FromDIP( 9, NULL);
     m_gradientType = wxAUI_GRADIENT_VERTICAL;
 
     InitBitmaps();
 }
 
+wxAuiDockArt* wxAuiDefaultDockArt::Clone()
+{
+    return new wxAuiDefaultDockArt(*this);
+}
+
 void
 wxAuiDefaultDockArt::InitBitmaps ()
 {
     // some built in bitmaps
+    // TODO: Provide x1.5 and x2.0 versions or migrate to SVG.
 #if defined( __WXMAC__ )
      static const unsigned char close_bits[]={
          0xFF, 0xFF, 0xFF, 0xFF, 0x0F, 0xFE, 0x03, 0xF8, 0x01, 0xF0, 0x19, 0xF3,
@@ -263,31 +272,63 @@ wxAuiDefaultDockArt::InitBitmaps ()
         0x7f,0xff,0xff,0xff,0xff,0xff,0xff,0xff};
 
 #ifdef __WXMAC__
-    m_inactiveCloseBitmap = wxAuiBitmapFromBits(close_bits, 16, 16, *wxWHITE);
-    m_activeCloseBitmap = wxAuiBitmapFromBits(close_bits, 16, 16, *wxWHITE );
+    const wxColour inactive = wxSystemSettings::GetColour(wxSYS_COLOUR_INACTIVECAPTION);
+    const wxColour active = wxSystemSettings::GetColour(wxSYS_COLOUR_CAPTIONTEXT);
 #else
-    m_inactiveCloseBitmap = wxAuiBitmapFromBits(close_bits, 16, 16, m_inactiveCaptionTextColour);
-    m_activeCloseBitmap = wxAuiBitmapFromBits(close_bits, 16, 16, m_activeCaptionTextColour);
+    const wxColor inactive = m_inactiveCaptionTextColour;
+    const wxColor active = m_activeCaptionTextColour;
 #endif
 
-#ifdef __WXMAC__
-    m_inactiveMaximizeBitmap = wxAuiBitmapFromBits(maximize_bits, 16, 16, *wxWHITE);
-    m_activeMaximizeBitmap = wxAuiBitmapFromBits(maximize_bits, 16, 16, *wxWHITE );
-#else
-    m_inactiveMaximizeBitmap = wxAuiBitmapFromBits(maximize_bits, 16, 16, m_inactiveCaptionTextColour);
-    m_activeMaximizeBitmap = wxAuiBitmapFromBits(maximize_bits, 16, 16, m_activeCaptionTextColour);
-#endif
+    m_inactiveCloseBitmap = wxAuiBitmapFromBits(close_bits, 16, 16, inactive);
+    m_activeCloseBitmap = wxAuiBitmapFromBits(close_bits, 16, 16, active);
 
-#ifdef __WXMAC__
-    m_inactiveRestoreBitmap = wxAuiBitmapFromBits(restore_bits, 16, 16, *wxWHITE);
-    m_activeRestoreBitmap = wxAuiBitmapFromBits(restore_bits, 16, 16, *wxWHITE );
-#else
-    m_inactiveRestoreBitmap = wxAuiBitmapFromBits(restore_bits, 16, 16, m_inactiveCaptionTextColour);
-    m_activeRestoreBitmap = wxAuiBitmapFromBits(restore_bits, 16, 16, m_activeCaptionTextColour);
-#endif
+    m_inactiveMaximizeBitmap = wxAuiBitmapFromBits(maximize_bits, 16, 16, inactive);
+    m_activeMaximizeBitmap = wxAuiBitmapFromBits(maximize_bits, 16, 16, active);
 
-    m_inactivePinBitmap = wxAuiBitmapFromBits(pin_bits, 16, 16, m_inactiveCaptionTextColour);
-    m_activePinBitmap = wxAuiBitmapFromBits(pin_bits, 16, 16, m_activeCaptionTextColour);
+    m_inactiveRestoreBitmap = wxAuiBitmapFromBits(restore_bits, 16, 16, inactive);
+    m_activeRestoreBitmap = wxAuiBitmapFromBits(restore_bits, 16, 16, active);
+
+    m_inactivePinBitmap = wxAuiBitmapFromBits(pin_bits, 16, 16, inactive);
+    m_activePinBitmap = wxAuiBitmapFromBits(pin_bits, 16, 16, active);
+}
+
+void wxAuiDefaultDockArt::UpdateColoursFromSystem()
+{
+    wxColor baseColour = wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE);
+
+    // the baseColour is too pale to use as our base colour,
+    // so darken it a bit --
+    if ((255-baseColour.Red()) +
+        (255-baseColour.Green()) +
+        (255-baseColour.Blue()) < 60)
+    {
+        baseColour = baseColour.ChangeLightness(92);
+    }
+
+    m_baseColour = baseColour;
+    wxColor darker1Colour = baseColour.ChangeLightness(85);
+    wxColor darker2Colour = baseColour.ChangeLightness(75);
+    wxColor darker3Colour = baseColour.ChangeLightness(60);
+    //wxColor darker4Colour = baseColour.ChangeLightness(50);
+    wxColor darker5Colour = baseColour.ChangeLightness(40);
+
+    m_activeCaptionColour = wxSystemSettings::GetColour(wxSYS_COLOUR_HIGHLIGHT);
+    m_activeCaptionGradientColour = wxAuiLightContrastColour(wxSystemSettings::GetColour(wxSYS_COLOUR_HIGHLIGHT));
+    m_activeCaptionTextColour = wxSystemSettings::GetColour(wxSYS_COLOUR_HIGHLIGHTTEXT);
+    m_inactiveCaptionColour = darker1Colour;
+    m_inactiveCaptionGradientColour = baseColour.ChangeLightness(97);
+    m_inactiveCaptionTextColour = wxSystemSettings::GetColour(wxSYS_COLOUR_INACTIVECAPTIONTEXT);
+
+    m_sashBrush = wxBrush(baseColour);
+    m_backgroundBrush = wxBrush(baseColour);
+    m_gripperBrush = wxBrush(baseColour);
+
+    m_borderPen = wxPen(darker2Colour);
+    int pen_width = wxWindow::FromDIP(1, NULL);
+    m_gripperPen1 = wxPen(darker5Colour, pen_width);
+    m_gripperPen2 = wxPen(darker3Colour, pen_width);
+    m_gripperPen3 = wxPen(*wxStockGDI::GetColour(wxStockGDI::COLOUR_WHITE), pen_width);
+    InitBitmaps();
 }
 
 int wxAuiDefaultDockArt::GetMetric(int id)
@@ -383,16 +424,25 @@ void wxAuiDefaultDockArt::DrawSash(wxDC& dc, wxWindow *window, int orientation,
     wxUnusedVar(window);
     wxUnusedVar(orientation);
 
-    HIRect splitterRect = CGRectMake( rect.x , rect.y , rect.width , rect.height );
-    CGContextRef cgContext ;
-    wxGCDCImpl *impl = (wxGCDCImpl*) dc.GetImpl();
-    cgContext = (CGContextRef) impl->GetGraphicsContext()->GetNativeContext() ;
+    if ( wxPlatformInfo::Get().CheckOSVersion(10, 14) && wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW).Red() < 128 )
+    {
+        dc.SetPen(*wxTRANSPARENT_PEN);
+        dc.SetBrush(m_sashBrush);
+        dc.DrawRectangle(rect.x, rect.y, rect.width, rect.height);
+    }
+    else
+    {
+        HIRect splitterRect = CGRectMake( rect.x , rect.y , rect.width , rect.height );
+        CGContextRef cgContext ;
+        wxGCDCImpl *impl = (wxGCDCImpl*) dc.GetImpl();
+        cgContext = (CGContextRef) impl->GetGraphicsContext()->GetNativeContext() ;
 
-    HIThemeSplitterDrawInfo drawInfo ;
-    drawInfo.version = 0 ;
-    drawInfo.state = kThemeStateActive ;
-    drawInfo.adornment = kHIThemeSplitterAdornmentNone ;
-    HIThemeDrawPaneSplitter( &splitterRect , &drawInfo , cgContext , kHIThemeOrientationNormal ) ;
+        HIThemeSplitterDrawInfo drawInfo ;
+        drawInfo.version = 0 ;
+        drawInfo.state = kThemeStateActive ;
+        drawInfo.adornment = kHIThemeSplitterAdornmentNone ;
+        HIThemeDrawPaneSplitter( &splitterRect , &drawInfo , cgContext , kHIThemeOrientationNormal ) ;
+    }
 
 #elif defined(__WXGTK__)
     // clear out the rectangle first
@@ -489,7 +539,7 @@ void wxAuiDefaultDockArt::DrawBorder(wxDC& dc, wxWindow* window, const wxRect& _
     {
         for (i = 0; i < border_width; ++i)
         {
-            dc.SetPen(*wxWHITE_PEN);
+            dc.SetPen(wxPen(wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW)));
             dc.DrawLine(rect.x, rect.y, rect.x+rect.width, rect.y);
             dc.DrawLine(rect.x, rect.y, rect.x, rect.y+rect.height);
             dc.SetPen(m_borderPen);
@@ -571,7 +621,8 @@ void wxAuiDefaultDockArt::DrawCaptionBackground(wxDC& dc, const wxRect& rect, bo
 }
 
 
-void wxAuiDefaultDockArt::DrawCaption(wxDC& dc, wxWindow *WXUNUSED(window),
+void wxAuiDefaultDockArt::DrawCaption(wxDC& dc,
+                                   wxWindow* window,
                                    const wxString& text,
                                    const wxRect& rect,
                                    wxAuiPaneInfo& pane)
@@ -585,9 +636,10 @@ void wxAuiDefaultDockArt::DrawCaption(wxDC& dc, wxWindow *WXUNUSED(window),
     int caption_offset = 0;
     if ( pane.icon.IsOk() )
     {
-        DrawIcon(dc, rect, pane);
+        DrawIcon(dc, window, rect, pane);
 
-        caption_offset += pane.icon.GetScaledWidth() + 3;
+        const wxBitmap& icon = pane.icon.GetBitmapFor(window);
+        caption_offset += icon.GetLogicalWidth() + window->FromDIP(3);
     }
 
     if (pane.state & wxAuiPaneInfo::optionActive)
@@ -600,8 +652,8 @@ void wxAuiDefaultDockArt::DrawCaption(wxDC& dc, wxWindow *WXUNUSED(window),
     dc.GetTextExtent(wxT("ABCDEFHXfgkj"), &w, &h);
 
     wxRect clip_rect = rect;
-    clip_rect.width -= 3; // text offset
-    clip_rect.width -= 2; // button padding
+    clip_rect.width -= window->FromDIP(3); // text offset
+    clip_rect.width -= window->FromDIP(2); // button padding
     if (pane.HasCloseButton())
         clip_rect.width -= m_buttonSize;
     if (pane.HasPinButton())
@@ -612,20 +664,43 @@ void wxAuiDefaultDockArt::DrawCaption(wxDC& dc, wxWindow *WXUNUSED(window),
     wxString draw_text = wxAuiChopText(dc, text, clip_rect.width);
 
     dc.SetClippingRegion(clip_rect);
-    dc.DrawText(draw_text, rect.x+3 + caption_offset, rect.y+(rect.height/2)-(h/2)-1);
+    dc.DrawText(draw_text, rect.x+window->FromDIP(3) + caption_offset, rect.y+(rect.height/2)-(h/2)-1);
     dc.DestroyClippingRegion();
 }
 
-void
-wxAuiDefaultDockArt::DrawIcon(wxDC& dc, const wxRect& rect, wxAuiPaneInfo& pane)
+#if WXWIN_COMPATIBILITY_3_0
+void wxAuiDefaultDockArt::DrawIcon(wxDC& dc, const wxRect& rect, wxAuiPaneInfo& pane)
 {
-   // Draw the icon centered vertically
-   dc.DrawBitmap(pane.icon,
-                 rect.x+2, rect.y+(rect.height-pane.icon.GetScaledHeight())/2,
-                 true);
+    DrawIcon(dc, NULL, rect, pane);
+}
+#endif
+
+void
+wxAuiDefaultDockArt::DrawIcon(wxDC& dc, wxWindow *window, const wxRect& rect, wxAuiPaneInfo& pane)
+{
+    if (!window)
+    {
+        window = wxTheApp->GetTopWindow();
+        wxCHECK_RET( window, "must have some window" );
+    }
+
+    // Ensure the icon fits into the title bar.
+    wxSize iconSize = pane.icon.GetPreferredLogicalSizeFor(window);
+    if (iconSize.y > rect.height)
+    {
+        iconSize *= static_cast<double>(rect.height) / iconSize.y;
+    }
+
+    // Draw the icon centered vertically
+    int xOffset = window->FromDIP(2);
+    const wxBitmap& icon = pane.icon.GetBitmap(window->ToPhys(iconSize));
+    dc.DrawBitmap(icon,
+                  rect.x+xOffset, rect.y+(rect.height-icon.GetLogicalHeight())/2,
+                  true);
 }
 
-void wxAuiDefaultDockArt::DrawGripper(wxDC& dc, wxWindow *WXUNUSED(window),
+void wxAuiDefaultDockArt::DrawGripper(wxDC& dc,
+                                   wxWindow* window,
                                    const wxRect& rect,
                                    wxAuiPaneInfo& pane)
 {
@@ -636,98 +711,97 @@ void wxAuiDefaultDockArt::DrawGripper(wxDC& dc, wxWindow *WXUNUSED(window),
 
     if (!pane.HasGripperTop())
     {
-        int y = 5;
+        int y = window->FromDIP(5);
         while (1)
         {
             dc.SetPen(m_gripperPen1);
-            dc.DrawPoint(rect.x+3, rect.y+y);
+            dc.DrawPoint(rect.x+window->FromDIP(3), rect.y+y);
             dc.SetPen(m_gripperPen2);
-            dc.DrawPoint(rect.x+3, rect.y+y+1);
-            dc.DrawPoint(rect.x+4, rect.y+y);
+            dc.DrawPoint(rect.x+window->FromDIP(3), rect.y+y+window->FromDIP(1));
+            dc.DrawPoint(rect.x+window->FromDIP(4), rect.y+y                   );
             dc.SetPen(m_gripperPen3);
-            dc.DrawPoint(rect.x+5, rect.y+y+1);
-            dc.DrawPoint(rect.x+5, rect.y+y+2);
-            dc.DrawPoint(rect.x+4, rect.y+y+2);
+            dc.DrawPoint(rect.x+window->FromDIP(5), rect.y+y+window->FromDIP(1));
+            dc.DrawPoint(rect.x+window->FromDIP(5), rect.y+y+window->FromDIP(2));
+            dc.DrawPoint(rect.x+window->FromDIP(4), rect.y+y+window->FromDIP(2));
 
-            y += 4;
-            if (y > rect.GetHeight()-5)
+            y += window->FromDIP(4);
+            if (y > rect.GetHeight()-window->FromDIP(5))
                 break;
         }
     }
     else
     {
-        int x = 5;
+        int x = window->FromDIP(5);
         while (1)
         {
             dc.SetPen(m_gripperPen1);
-            dc.DrawPoint(rect.x+x, rect.y+3);
+            dc.DrawPoint(rect.x+x, rect.y+window->FromDIP(3));
             dc.SetPen(m_gripperPen2);
-            dc.DrawPoint(rect.x+x+1, rect.y+3);
-            dc.DrawPoint(rect.x+x, rect.y+4);
+            dc.DrawPoint(rect.x+x+window->FromDIP(1), rect.y+window->FromDIP(3));
+            dc.DrawPoint(rect.x+x                   , rect.y+window->FromDIP(4));
             dc.SetPen(m_gripperPen3);
-            dc.DrawPoint(rect.x+x+1, rect.y+5);
-            dc.DrawPoint(rect.x+x+2, rect.y+5);
-            dc.DrawPoint(rect.x+x+2, rect.y+4);
+            dc.DrawPoint(rect.x+x+window->FromDIP(1), rect.y+window->FromDIP(5));
+            dc.DrawPoint(rect.x+x+window->FromDIP(2), rect.y+window->FromDIP(5));
+            dc.DrawPoint(rect.x+x+window->FromDIP(2), rect.y+window->FromDIP(4));
 
-            x += 4;
-            if (x > rect.GetWidth()-5)
+            x += window->FromDIP(4);
+            if (x > rect.GetWidth()-window->FromDIP(5))
                 break;
         }
     }
 }
 
-void wxAuiDefaultDockArt::DrawPaneButton(wxDC& dc, wxWindow *WXUNUSED(window),
+void wxAuiDefaultDockArt::DrawPaneButton(wxDC& dc,
+                                      wxWindow* window,
                                       int button,
                                       int button_state,
                                       const wxRect& _rect,
                                       wxAuiPaneInfo& pane)
 {
-    wxBitmap bmp;
+    wxBitmapBundle bb;
     switch (button)
     {
         default:
         case wxAUI_BUTTON_CLOSE:
             if (pane.state & wxAuiPaneInfo::optionActive)
-                bmp = m_activeCloseBitmap;
+                bb = m_activeCloseBitmap;
             else
-                bmp = m_inactiveCloseBitmap;
+                bb = m_inactiveCloseBitmap;
             break;
         case wxAUI_BUTTON_PIN:
             if (pane.state & wxAuiPaneInfo::optionActive)
-                bmp = m_activePinBitmap;
+                bb = m_activePinBitmap;
             else
-                bmp = m_inactivePinBitmap;
+                bb = m_inactivePinBitmap;
             break;
         case wxAUI_BUTTON_MAXIMIZE_RESTORE:
             if (pane.IsMaximized())
             {
                 if (pane.state & wxAuiPaneInfo::optionActive)
-                    bmp = m_activeRestoreBitmap;
+                    bb = m_activeRestoreBitmap;
                 else
-                    bmp = m_inactiveRestoreBitmap;
+                    bb = m_inactiveRestoreBitmap;
             }
             else
             {
                 if (pane.state & wxAuiPaneInfo::optionActive)
-                    bmp = m_activeMaximizeBitmap;
+                    bb = m_activeMaximizeBitmap;
                 else
-                    bmp = m_inactiveMaximizeBitmap;
+                    bb = m_inactiveMaximizeBitmap;
             }
             break;
     }
 
+    const wxBitmap& bmp = bb.GetBitmapFor(window);
 
     wxRect rect = _rect;
 
-    int old_y = rect.y;
-    rect.y = rect.y + (rect.height/2) - (bmp.GetScaledHeight()/2);
-    rect.height = old_y + rect.height - rect.y - 1;
-
+    rect.y = rect.y + (rect.height/2) - (bmp.GetLogicalHeight()/2);
 
     if (button_state == wxAUI_BUTTON_STATE_PRESSED)
     {
-        rect.x++;
-        rect.y++;
+        rect.x += window->FromDIP(1);
+        rect.y += window->FromDIP(1);
     }
 
     if (button_state == wxAUI_BUTTON_STATE_HOVER ||
@@ -745,10 +819,11 @@ void wxAuiDefaultDockArt::DrawPaneButton(wxDC& dc, wxWindow *WXUNUSED(window),
         }
 
         // draw the background behind the button
-        dc.DrawRectangle(rect.x, rect.y, 15, 15);
+        dc.DrawRectangle(rect.x, rect.y,
+            bmp.GetLogicalWidth() - window->FromDIP(1),
+            bmp.GetLogicalHeight() - window->FromDIP(1));
     }
 
-
     // draw the button itself
     dc.DrawBitmap(bmp, rect.x, rect.y, true);
 }
