@@ -9,9 +9,6 @@
 // For compilers that support precompilation, includes "wx.h".
 #include "wx/wxprec.h"
 
-#ifdef __BORLANDC__
-    #pragma hdrstop
-#endif
 
 #if wxUSE_IMAGE
 
@@ -521,8 +518,24 @@ bool wxBMPHandler::DoLoadDib(wxImage * image, int width, int height,
     // allocate space for palette if needed:
     BMPPalette *cmap;
 
-    if ( bpp < 16 )
+    if ( bpp <= 8 )
     {
+        // The bit depth is 8bpp, 4bpp, or 1bpp, which means that ncolors is
+        // the size of a palette.  The largest useful palette is 256 since
+        // anything larger couldn't be referenced by a pixel.  Since ncolors
+        // comes from the file, which could be corrupt or malicious, reject
+        // any bitmaps that have a dubious palette size.
+        if ( ncolors < 0 || 256 < ncolors )
+        {
+            if ( verbose )
+            {
+                wxLogError(
+                    _("BMP: header has biClrUsed=%d when biBitCount=%d."),
+                    ncolors, bpp);
+            }
+            return false;
+        }
+
         cmap = new BMPPalette[ncolors];
         if ( !cmap )
         {
@@ -586,10 +599,12 @@ bool wxBMPHandler::DoLoadDib(wxImage * image, int width, int height,
     // Reading the palette, if it exists:
     if ( bpp < 16 && ncolors != 0 )
     {
+#if wxUSE_PALETTE
         wxScopedArray<unsigned char>
              r(ncolors),
              g(ncolors),
              b(ncolors);
+#endif // wxUSE_PALETTE
         for (int j = 0; j < ncolors; j++)
         {
             if (hasPalette)
@@ -600,18 +615,19 @@ bool wxBMPHandler::DoLoadDib(wxImage * image, int width, int height,
                 cmap[j].b = bbuf[0];
                 cmap[j].g = bbuf[1];
                 cmap[j].r = bbuf[2];
-
-                r[j] = cmap[j].r;
-                g[j] = cmap[j].g;
-                b[j] = cmap[j].b;
             }
             else
             {
                 //used in reading .ico file mask
-                r[j] = cmap[j].r =
-                g[j] = cmap[j].g =
-                b[j] = cmap[j].b = ( j ? 255 : 0 );
+                cmap[j].r =
+                cmap[j].g =
+                cmap[j].b = ( j ? 255 : 0 );
             }
+#if wxUSE_PALETTE
+            r[j] = cmap[j].r;
+            g[j] = cmap[j].g;
+            b[j] = cmap[j].b;
+#endif // wxUSE_PALETTE
         }
 
 #if wxUSE_PALETTE
@@ -756,11 +772,15 @@ bool wxBMPHandler::DoLoadDib(wxImage * image, int width, int height,
 
                         if ( first == 0 )
                         {
+                            // This is an escape sequence with special meaning.
                             if ( aByte == 0 )
                             {
                                 // end of scanline marker
-                                column = width;
-                                row--;
+                                // This is ignored if the end-of-line was
+                                // implicitly assumed when column==width,
+                                // in which case column is now 0.
+                                if (column != 0)
+                                    column = width;
                             }
                             else if ( aByte == 1 )
                             {
@@ -771,18 +791,23 @@ bool wxBMPHandler::DoLoadDib(wxImage * image, int width, int height,
                             else if ( aByte == 2 )
                             {
                                 // delta marker, move in image
+
+                                // process column offset
                                 aByte = stream.GetC();
                                 if ( !stream.IsOk() )
                                     return false;
                                 column += aByte;
-                                linepos = column * bpp / 4;
+
+                                // process row offset
                                 aByte = stream.GetC();
                                 if ( !stream.IsOk() )
                                     return false;
-                                row += aByte; // upside down
+                                row += aByte;
+                                line = isUpsideDown ? height - 1 - row : row;
                             }
                             else
                             {
+                                // absolute mode (pixels not runs)
                                 int absolute = aByte;
                                 wxUint8 nibble[2] ;
                                 int readBytes = 0 ;
@@ -801,8 +826,6 @@ bool wxBMPHandler::DoLoadDib(wxImage * image, int width, int height,
                                     ptr[poffset + 1] = cmap[nibble[k%2]].g;
                                     ptr[poffset + 2] = cmap[nibble[k%2]].b;
                                     column++;
-                                    if ( k % 2 )
-                                        linepos++;
                                 }
                                 if ( readBytes & 0x01 )
                                 {
@@ -824,8 +847,6 @@ bool wxBMPHandler::DoLoadDib(wxImage * image, int width, int height,
                                 ptr[poffset + 1] = cmap[nibble[l%2]].g;
                                 ptr[poffset + 2] = cmap[nibble[l%2]].b;
                                 column++;
-                                if ( l % 2 )
-                                    linepos++;
                             }
                         }
                     }
@@ -858,8 +879,11 @@ bool wxBMPHandler::DoLoadDib(wxImage * image, int width, int height,
                             if ( aByte == 0 )
                             {
                                 // end of scanline marker
-                                column = width;
-                                row--;
+                                // This is ignored if the end-of-line was
+                                // implicitly assumed when column==width,
+                                // in which case column is now 0.
+                                if (column != 0)
+                                    column = width;
                             }
                             else if ( aByte == 1 )
                             {
@@ -870,22 +894,26 @@ bool wxBMPHandler::DoLoadDib(wxImage * image, int width, int height,
                             else if ( aByte == 2 )
                             {
                                 // delta marker, move in image
+
+                                // process column offset
                                 aByte = stream.GetC();
                                 if ( !stream.IsOk() )
                                     return false;
                                 column += aByte;
-                                linepos = column * bpp / 8;
+
+                                // process row offset
                                 aByte = stream.GetC();
                                 if ( !stream.IsOk() )
                                     return false;
-                                row -= aByte;
+                                row += aByte;
+                                line = isUpsideDown ? height - 1 - row : row;
                             }
                             else
                             {
+                                // absolute mode (pixels not runs)
                                 int absolute = aByte;
                                 for (int k = 0; k < absolute; k++)
                                 {
-                                    linepos++;
                                     aByte = stream.GetC();
                                     if ( !stream.IsOk() )
                                         return false;
@@ -904,13 +932,13 @@ bool wxBMPHandler::DoLoadDib(wxImage * image, int width, int height,
                         }
                         else
                         {
+                            // encoded mode (repeat aByte first times)
                             for ( int l = 0; l < first && column < width; l++ )
                             {
                                 ptr[poffset    ] = cmap[aByte].r;
                                 ptr[poffset + 1] = cmap[aByte].g;
                                 ptr[poffset + 2] = cmap[aByte].b;
                                 column++;
-                                linepos++;
                             }
                         }
                     }
@@ -978,7 +1006,7 @@ bool wxBMPHandler::DoLoadDib(wxImage * image, int width, int height,
                 column++;
             }
         }
-        while ( (linepos < linesize) && (comp != 1) && (comp != 2) )
+        while ( (linepos < linesize) && (comp != BI_RLE8) && (comp != BI_RLE4) )
         {
             ++linepos;
             if ( !stream.ReadAll(&aByte, 1) )
@@ -1004,7 +1032,6 @@ bool wxBMPHandler::LoadDib(wxImage *image, wxInputStream& stream,
 {
     wxUint16        aWord;
     wxInt32         dbuf[4];
-    wxInt8          bbuf[4];
 
     // offset to bitmap data
     wxFileOffset offset;
@@ -1012,6 +1039,7 @@ bool wxBMPHandler::LoadDib(wxImage *image, wxInputStream& stream,
     wxInt32 hdrSize;
     if ( IsBmp )
     {
+        wxInt8 bbuf[4];
         // read the header off the .BMP format file
         if ( !stream.ReadAll(bbuf, 2) ||
              !stream.ReadAll(dbuf, 16) )
@@ -1057,7 +1085,7 @@ bool wxBMPHandler::LoadDib(wxImage *image, wxInputStream& stream,
         width = wxINT32_SWAP_ON_BE((int)dbuf[0]);
         height = wxINT32_SWAP_ON_BE((int)dbuf[1]);
     }
-    if ( !IsBmp)height = height  / 2; // for icons divide by 2
+    if ( !IsBmp) height /= 2; // for icons divide by 2
 
     if ( width > 32767 )
     {
@@ -1156,6 +1184,18 @@ bool wxBMPHandler::LoadDib(wxImage *image, wxInputStream& stream,
 
         ncolors = wxINT32_SWAP_ON_BE( (int)dbuf[0] );
         res.Init(dbuf[2]/100, dbuf[3]/100);
+
+        // We've read BITMAPINFOHEADER data but for BITMAPV4HEADER or BITMAPV5HEADER
+        // we have to forward stream position to after the actual bitmap header.
+        //
+        // Note: hardcode its size as struct BITMAPINFOHEADER is not defined on
+        // non-MSW platforms.
+        const wxInt32 sizeBITMAPINFOHEADER = 40;
+        if ( hdrSize > sizeBITMAPINFOHEADER )
+        {
+            if ( stream.SeekI(hdrSize - sizeBITMAPINFOHEADER, wxFromCurrent) == wxInvalidOffset )
+                return false;
+        }
     }
     if (ncolors == 0)
         ncolors = 1 << bpp;
@@ -1368,7 +1408,7 @@ bool wxICOHandler::SaveFile(wxImage *image,
         }
         image->SetOption(wxIMAGE_OPTION_BMP_FORMAT, bppFormat);
 
-        // monochome bitmap:
+        // monochrome bitmap:
         mask.SetOption(wxIMAGE_OPTION_BMP_FORMAT, wxBMP_1BPP_BW);
         bool IsBmp = false;
         bool IsMask = false;
