@@ -34,6 +34,7 @@
     #include "wx/dcclient.h"
     #include "wx/stattext.h"
     #include "wx/intl.h"
+    #include "wx/textctrl.h"
     #include "wx/textdlg.h"
     #include "wx/sizer.h"
     #include "wx/module.h"
@@ -44,6 +45,7 @@
 #include "wx/print.h"
 #include "wx/dcprint.h"
 #include "wx/artprov.h"
+#include "wx/display.h"
 
 #include <stdlib.h>
 #include <string.h>
@@ -637,7 +639,7 @@ bool wxPrintout::SetUp(wxDC& dc)
 {
     wxCHECK_MSG( dc.IsOk(), false, "should have a valid DC to set up" );
 
-    SetPPIScreen(wxGetDisplayPPI());
+    SetPPIScreen(wxDisplay::GetStdPPI());
 
     // We need to know printer PPI. In most ports, this can be retrieved from
     // the printer DC, but in others it is computed (probably for legacy
@@ -934,6 +936,10 @@ wxScrolledWindow(parent, wxID_ANY, pos, size, style | wxFULL_REPAINT_ON_RESIZE,
     SetBackgroundColour(wxSystemSettings::GetColour(colourIndex));
 
     SetScrollbars(10, 10, 100, 100);
+
+    // Use some reasonable default size for this window, roughly proportional
+    // to the paper sheet.
+    SetInitialSize(wxSize(600, 750));
 }
 
 wxPreviewCanvas::~wxPreviewCanvas()
@@ -1058,7 +1064,7 @@ void wxPreviewCanvas::OnMouseWheel(wxMouseEvent& event)
             else
                 delta = 50;
 
-            if ( event.GetWheelRotation() > 0 )
+            if ( event.GetWheelRotation() < 0 )
                 delta = -delta;
 
             int newZoom = currentZoom + delta;
@@ -1484,7 +1490,6 @@ public:
     ~SizerWithButtons()
     {
         m_parent->SetSizer(m_sizer);
-        m_sizer->Fit(m_parent);
     }
 
 
@@ -1495,7 +1500,7 @@ public:
         {
             m_needsSeparator = false;
 
-            m_sizer->AddSpacer(2*wxSizerFlags::GetDefaultBorder());
+            m_sizer->AddSpacer(wxRound(2*wxSizerFlags::GetDefaultBorderFractional()));
         }
 
         m_hasContents = true;
@@ -1523,9 +1528,13 @@ public:
     // as everything else added after it will be added on the right side too.
     void AddAtEnd(wxWindow *win)
     {
+        // Ensure there is at least the same gap before the final button as
+        // between the other groups.
+        m_sizer->AddSpacer(wxRound(2*wxSizerFlags::GetDefaultBorderFractional()));
+
         m_sizer->AddStretchSpacer();
         m_sizer->Add(win,
-                     wxSizerFlags().Border(wxTOP | wxBOTTOM | wxRIGHT).Center());
+                     wxSizerFlags().Border().Center());
     }
 
     // Indicates the end of a group of buttons, a separator will be added after
@@ -1612,7 +1621,7 @@ void wxPreviewControlBar::CreateButtons()
         };
         int n = WXSIZEOF(choices);
 
-        m_zoomControl = new wxChoice( this, wxID_PREVIEW_ZOOM, wxDefaultPosition, wxSize(70,wxDefaultCoord), n, choices, 0 );
+        m_zoomControl = new wxChoice( this, wxID_PREVIEW_ZOOM, wxDefaultPosition, wxDefaultSize, n, choices, 0 );
         sizer.Add(m_zoomControl);
         SetZoomControl(m_printPreview->GetZoom());
 
@@ -1693,7 +1702,8 @@ void wxPreviewFrame::OnChar(wxKeyEvent &event)
 
 wxPreviewFrame::wxPreviewFrame(wxPrintPreviewBase *preview, wxWindow *parent, const wxString& title,
                                const wxPoint& pos, const wxSize& size, long style, const wxString& name):
-wxFrame(parent, wxID_ANY, title, pos, size, style, name)
+wxFrame(parent, wxID_ANY, title, pos, size, style, name),
+    m_initialSize(size)
 {
     m_printPreview = preview;
     m_controlBar = NULL;
@@ -1760,12 +1770,27 @@ void wxPreviewFrame::InitializeWithModality(wxPreviewFrameModalityKind kind)
 
     wxBoxSizer* const sizer = new wxBoxSizer( wxVERTICAL );
 
-    sizer->Add( m_controlBar, wxSizerFlags().Expand().Border() );
-    sizer->Add( m_previewCanvas, wxSizerFlags(1).Expand().Border() );
+    sizer->Add( m_controlBar, wxSizerFlags().Expand() );
+    sizer->Add( m_previewCanvas, wxSizerFlags(1).Expand() );
 
-    SetAutoLayout( true );
     SetSizer( sizer );
 
+    // Respect the user-specified size, if any, but use the best appropriate
+    // size by default if none was explicitly given.
+    if ( !m_initialSize.IsFullySpecified() )
+    {
+        wxSize size = m_initialSize;
+        size.SetDefaults(sizer->ComputeFittingWindowSize(this));
+        SetSize(size);
+    }
+
+    // We don't want to restrict shrinking the window vertically as it might be
+    // too tall (see SetInitialSize() call in wxPreviewCanvas ctor), but we do
+    // want to make it at least as wide as the control bar, as otherwise the
+    // buttons wouldn't fit, and restricting it to at least its height
+    // vertically is also quite reasonable.
+    SetSizeHints(ClientToWindowSize(m_controlBar->GetBestSize()));
+
     m_modalityKind = kind;
     switch ( m_modalityKind )
     {
@@ -1793,8 +1818,6 @@ void wxPreviewFrame::InitializeWithModality(wxPreviewFrameModalityKind kind)
         SetWindowStyle((GetWindowStyle() & ~wxMINIMIZE_BOX) | wxFRAME_NO_TASKBAR);
     }
 
-    Layout();
-
     m_printPreview->AdjustScrollbars(m_previewCanvas);
     m_previewCanvas->SetFocus();
     m_controlBar->SetFocus();
@@ -1858,7 +1881,7 @@ void wxPrintPreviewBase::Init(wxPrintout *printout,
     m_currentPage = 1;
     m_currentZoom = 70;
     m_topMargin =
-    m_leftMargin = 2*wxSizerFlags::GetDefaultBorder();
+    m_leftMargin = wxRound(2*wxSizerFlags::GetDefaultBorderFractional());
     m_pageWidth = 0;
     m_pageHeight = 0;
     m_printingPrepared = false;
