@@ -41,6 +41,7 @@
 
 wxDEFINE_EVENT( wxEVT_SPLITTER_SASH_POS_CHANGED, wxSplitterEvent );
 wxDEFINE_EVENT( wxEVT_SPLITTER_SASH_POS_CHANGING, wxSplitterEvent );
+wxDEFINE_EVENT( wxEVT_SPLITTER_SASH_POS_RESIZE, wxSplitterEvent);
 wxDEFINE_EVENT( wxEVT_SPLITTER_DOUBLECLICKED, wxSplitterEvent );
 wxDEFINE_EVENT( wxEVT_SPLITTER_UNSPLIT, wxSplitterEvent );
 
@@ -60,6 +61,7 @@ wxIMPLEMENT_DYNAMIC_CLASS(wxSplitterEvent, wxNotifyEvent);
 wxBEGIN_EVENT_TABLE(wxSplitterWindow, wxWindow)
     EVT_PAINT(wxSplitterWindow::OnPaint)
     EVT_SIZE(wxSplitterWindow::OnSize)
+    EVT_DPI_CHANGED(wxSplitterWindow::OnDPIChanged)
     EVT_MOUSE_EVENTS(wxSplitterWindow::OnMouseEvent)
     EVT_MOUSE_CAPTURE_LOST(wxSplitterWindow::OnMouseCaptureLost)
 
@@ -311,7 +313,7 @@ void wxSplitterWindow::OnMouseEvent(wxMouseEvent& event)
                 OnUnsplit(removedWindow);
                 wxSplitterEvent eventUnsplit(wxEVT_SPLITTER_UNSPLIT, this);
                 eventUnsplit.m_data.win = removedWindow;
-                (void)DoSendEvent(eventUnsplit);
+                (void)ProcessWindowEvent(eventUnsplit);
                 SetSashPositionAndNotify(0);
             }
             else if ( posSashNew == GetWindowSize() )
@@ -322,7 +324,7 @@ void wxSplitterWindow::OnMouseEvent(wxMouseEvent& event)
                 OnUnsplit(removedWindow);
                 wxSplitterEvent eventUnsplit(wxEVT_SPLITTER_UNSPLIT, this);
                 eventUnsplit.m_data.win = removedWindow;
-                (void)DoSendEvent(eventUnsplit);
+                (void)ProcessWindowEvent(eventUnsplit);
                 SetSashPositionAndNotify(0);
             }
             else
@@ -471,6 +473,8 @@ void wxSplitterWindow::OnSize(wxSizeEvent& event)
 
             // Apply gravity if we use it.
             int delta = (int) ( (size - old_size)*m_sashGravity );
+
+            // If delta == 0 then sash will be set according to the windows min size.
             if ( delta != 0 )
             {
                 newPosition = m_sashPosition + delta;
@@ -478,6 +482,31 @@ void wxSplitterWindow::OnSize(wxSizeEvent& event)
                     newPosition = m_minimumPaneSize;
             }
 
+            // Send an event with the newly calculated position. The handler
+            // can then override the new position by setting the new position.
+            wxSplitterEvent update(wxEVT_SPLITTER_SASH_POS_RESIZE, this);
+            update.m_data.resize.pos = newPosition;
+            update.m_data.resize.oldSize = old_size;
+            update.m_data.resize.newSize = size;
+
+            if ( ProcessWindowEvent(update) )
+            {
+                if (update.IsAllowed())
+                {
+                    // If the user set the sashposition to -1
+                    // we keep the already calculated value,
+                    // otherwise the user provided the new position.
+                    int userPos = update.GetSashPosition();
+                    if (userPos != -1)
+                        newPosition = userPos;
+                }
+                else
+                {
+                    // the event handler vetoed the change
+                    newPosition = -1;
+                }
+            }
+
             // Also check if the second window became too small.
             newPosition = AdjustSashPosition(newPosition == -1
                                                  ? m_sashPosition
@@ -492,6 +521,15 @@ void wxSplitterWindow::OnSize(wxSizeEvent& event)
     SizeWindows();
 }
 
+void wxSplitterWindow::OnDPIChanged(wxDPIChangedEvent& event)
+{
+    m_minimumPaneSize = event.ScaleX(m_minimumPaneSize);
+    m_sashPosition = event.ScaleX(m_sashPosition);
+    m_lastSize = event.Scale(m_lastSize);
+
+    event.Skip();
+}
+
 void wxSplitterWindow::SetSashGravity(double gravity)
 {
     wxCHECK_RET( gravity >= 0. && gravity <= 1.,
@@ -669,9 +707,9 @@ void wxSplitterWindow::SetSashPositionAndNotify(int sashPos)
     DoSetSashPosition(sashPos);
 
     wxSplitterEvent event(wxEVT_SPLITTER_SASH_POS_CHANGED, this);
-    event.m_data.pos = m_sashPosition;
+    event.m_data.resize.pos = m_sashPosition;
 
-    (void)DoSendEvent(event);
+    (void)ProcessWindowEvent(event);
 }
 
 // Position and size subwindows.
@@ -898,11 +936,6 @@ void wxSplitterWindow::UpdateSize()
     SizeWindows();
 }
 
-bool wxSplitterWindow::DoSendEvent(wxSplitterEvent& event)
-{
-    return !GetEventHandler()->ProcessEvent(event) || event.IsAllowed();
-}
-
 wxSize wxSplitterWindow::DoGetBestSize() const
 {
     // get best sizes of subwindows
@@ -1005,17 +1038,20 @@ int wxSplitterWindow::OnSashPositionChanging(int newSashPosition)
     // FIXME: shouldn't we do it before the adjustments above so as to ensure
     //        that the sash position is always reasonable?
     wxSplitterEvent event(wxEVT_SPLITTER_SASH_POS_CHANGING, this);
-    event.m_data.pos = newSashPosition;
+    event.m_data.resize.pos = newSashPosition;
 
-    if ( !DoSendEvent(event) )
+    if ( ProcessWindowEvent(event) )
     {
-        // the event handler vetoed the change
-        newSashPosition = -1;
-    }
-    else
-    {
-        // it could have been changed by it
-        newSashPosition = event.GetSashPosition();
+        if (event.IsAllowed())
+        {
+            // it could have been changed by it
+            newSashPosition = event.GetSashPosition();
+        }
+        else
+        {
+            // the event handler vetoed the change
+            newSashPosition = -1;
+        }
     }
 
     return newSashPosition;
@@ -1031,7 +1067,7 @@ void wxSplitterWindow::OnDoubleClickSash(int x, int y)
     wxSplitterEvent event(wxEVT_SPLITTER_DOUBLECLICKED, this);
     event.m_data.pt.x = x;
     event.m_data.pt.y = y;
-    if ( DoSendEvent(event) )
+    if ( !ProcessWindowEvent(event) || event.IsAllowed() )
     {
         if ( GetMinimumPaneSize() == 0 || m_permitUnsplitAlways )
         {
@@ -1040,7 +1076,7 @@ void wxSplitterWindow::OnDoubleClickSash(int x, int y)
             {
                 wxSplitterEvent unsplitEvent(wxEVT_SPLITTER_UNSPLIT, this);
                 unsplitEvent.m_data.win = win;
-                (void)DoSendEvent(unsplitEvent);
+                (void)ProcessWindowEvent(unsplitEvent);
             }
         }
     }
