@@ -16,8 +16,10 @@
     #include "wx/image.h"
     #include "wx/colour.h"
     #include "wx/cursor.h"
+    #include "wx/dc.h"
 #endif
 
+#include "wx/math.h"
 #include "wx/rawbmp.h"
 
 #include "wx/gtk/private/object.h"
@@ -58,7 +60,6 @@ static void MaskToAlpha(GdkPixmap* mask, GdkPixbuf* pixbuf, int w, int h)
     {
         for (int x = w; x; x--, p += 4, mask_data += 3)
         {
-            *p = 255;
             // no need to test all 3 components,
             //   pixels are either (0,0,0) or (0xff,0xff,0xff)
             if (mask_data[0] == 0)
@@ -208,7 +209,7 @@ bool wxMask::InitFromColour(const wxBitmap& bitmap, const wxColour& colour)
         const wxByte r_mask = colour.Red();
         const wxByte g_mask = colour.Green();
         const wxByte b_mask = colour.Blue();
-        GdkPixbuf* pixbuf = bitmap.GetPixbuf();
+        GdkPixbuf* pixbuf = bitmap.GetPixbufNoMask();
         const wxByte* in = gdk_pixbuf_get_pixels(pixbuf);
         const int inc = 3 + int(gdk_pixbuf_get_has_alpha(pixbuf) != 0);
         const int rowpadding = gdk_pixbuf_get_rowstride(pixbuf) - inc * w;
@@ -333,7 +334,6 @@ public:
     virtual bool IsOk() const wxOVERRIDE;
 
 #ifdef __WXGTK3__
-    GdkPixbuf* m_pixbufMask;
     GdkPixbuf* m_pixbufNoMask;
     cairo_surface_t* m_surface;
     double m_scaleFactor;
@@ -341,6 +341,7 @@ public:
     GdkPixmap      *m_pixmap;
     GdkPixbuf      *m_pixbuf;
 #endif
+    GdkPixbuf      *m_pixbufMask;
     wxMask         *m_mask;
     int             m_width;
     int             m_height;
@@ -359,7 +360,6 @@ public:
 wxBitmapRefData::wxBitmapRefData(int width, int height, int depth)
 {
 #ifdef __WXGTK3__
-    m_pixbufMask = NULL;
     m_pixbufNoMask = NULL;
     m_surface = NULL;
     m_scaleFactor = 1;
@@ -367,6 +367,7 @@ wxBitmapRefData::wxBitmapRefData(int width, int height, int depth)
     m_pixmap = NULL;
     m_pixbuf = NULL;
 #endif
+    m_pixbufMask = NULL;
     m_mask = NULL;
     m_width = width;
     m_height = height;
@@ -384,13 +385,13 @@ wxBitmapRefData::wxBitmapRefData(int width, int height, int depth)
 wxBitmapRefData::~wxBitmapRefData()
 {
 #ifdef __WXGTK3__
-    if (m_pixbufMask)
-        g_object_unref(m_pixbufMask);
     if (m_pixbufNoMask)
         g_object_unref(m_pixbufNoMask);
     if (m_surface)
         cairo_surface_destroy(m_surface);
 #else
+    if (m_pixbufMask)
+        g_object_unref(m_pixbufMask);
     if (m_pixmap)
         g_object_unref (m_pixmap);
     if (m_pixbuf)
@@ -542,7 +543,6 @@ bool wxBitmap::Create( int width, int height, int depth )
     return true;
 }
 
-#ifdef __WXGTK3__
 static void CopyImageData(
     guchar* dst, int dstChannels, int dstStride,
     const guchar* src, int srcChannels, int srcStride,
@@ -587,11 +587,10 @@ static void CopyImageData(
         }
     }
 }
-#endif
 
 #if wxUSE_IMAGE
 #ifdef __WXGTK3__
-wxBitmap::wxBitmap(const wxImage& image, int depth, double scale)
+void wxBitmap::InitFromImage(const wxImage& image, int depth, double scale)
 {
     wxCHECK_RET(image.IsOk(), "invalid image");
 
@@ -638,7 +637,7 @@ wxBitmap::wxBitmap(const wxImage& image, int depth, double scale)
     }
 }
 #else
-wxBitmap::wxBitmap(const wxImage& image, int depth, double WXUNUSED(scale))
+void wxBitmap::InitFromImage(const wxImage& image, int depth, double WXUNUSED(scale))
 {
     wxCHECK_RET(image.IsOk(), "invalid image");
 
@@ -759,10 +758,41 @@ bool wxBitmap::CreateFromImageAsPixbuf(const wxImage& image)
         }
     }
 
+    if ( image.HasMask() )
+    {
+        const size_t out_size = size_t((width + 7) / 8) * unsigned(height);
+        wxByte* out = new wxByte[out_size];
+        memset(out, 0xff, out_size);
+        const wxByte r_mask = image.GetMaskRed();
+        const wxByte g_mask = image.GetMaskGreen();
+        const wxByte b_mask = image.GetMaskBlue();
+        const wxByte* in = image.GetData();
+        unsigned bit_index = 0;
+        for (int y = 0; y < height; y++)
+        {
+            for (int x = 0; x < width; x++, in += 3, bit_index++)
+                if (in[0] == r_mask && in[1] == g_mask && in[2] == b_mask)
+                    out[bit_index >> 3] ^= 1 << (bit_index & 7);
+            bit_index = (bit_index + 7) & ~7u;
+        }
+        SetMask(new wxMask(gdk_bitmap_create_from_data(wxGetTopLevelGDK(), reinterpret_cast<char*>(out), width, height)));
+        delete[] out;
+    }
+
     return true;
 }
 #endif
 
+wxBitmap::wxBitmap(const wxImage& image, int depth, double scale)
+{
+    InitFromImage(image, depth, scale);
+}
+
+wxBitmap::wxBitmap(const wxImage& image, const wxDC& dc)
+{
+    InitFromImage(image, -1, dc.GetContentScaleFactor());
+}
+
 wxImage wxBitmap::ConvertToImage() const
 {
 #ifdef __WXGTK3__
@@ -839,7 +869,7 @@ wxImage wxBitmap::ConvertToImage() const
     // prefer pixbuf if available, it will preserve alpha and should be quicker
     if (HasPixbuf())
     {
-        GdkPixbuf *pixbuf = GetPixbuf();
+        GdkPixbuf *pixbuf = GetPixbufNoMask();
         unsigned char* alpha = NULL;
         if (gdk_pixbuf_get_has_alpha(pixbuf))
         {
@@ -886,8 +916,8 @@ wxImage wxBitmap::ConvertToImage() const
         if (pixmap_invert != NULL)
             g_object_unref(pixmap_invert);
     }
-    // convert mask, unless there is already alpha
-    if (GetMask() && !image.HasAlpha())
+    // convert mask, even there is already alpha. Image can have both.
+    if ( GetMask() )
     {
         // we hard code the mask colour for now but we could also make an
         // effort (and waste time) to choose a colour not present in the
@@ -963,22 +993,40 @@ void wxBitmap::SetMask( wxMask *mask )
     AllocExclusive();
     delete M_BMPDATA->m_mask;
     M_BMPDATA->m_mask = mask;
-}
-
-bool wxBitmap::CopyFromIcon(const wxIcon& icon)
-{
-    *this = icon;
-    return IsOk();
+    if (M_BMPDATA->m_pixbufMask)
+    {
+        g_object_unref(M_BMPDATA->m_pixbufMask);
+        M_BMPDATA->m_pixbufMask = NULL;
+    }
 }
 
 #ifdef __WXGTK3__
-bool wxBitmap::CreateScaled(int w, int h, int depth, double scale)
+bool wxBitmap::Create(int width, int height, const wxDC& dc)
 {
-    Create(int(w * scale), int(h * scale), depth);
+    return DoCreate(wxSize(width, height),
+                    dc.GetContentScaleFactor(),
+                    wxBITMAP_SCREEN_DEPTH);
+}
+
+bool wxBitmap::DoCreate(const wxSize& size, double scale, int depth)
+{
+    Create(size*scale, depth);
     M_BMPDATA->m_scaleFactor = scale;
     return true;
 }
 
+void wxBitmap::SetScaleFactor(double scale)
+{
+    wxCHECK_RET(m_refData, "invalid bitmap");
+
+    if ( M_BMPDATA->m_scaleFactor != scale )
+    {
+        AllocExclusive();
+
+        M_BMPDATA->m_scaleFactor = scale;
+    }
+}
+
 double wxBitmap::GetScaleFactor() const
 {
     wxCHECK_MSG(m_refData, -1, "invalid bitmap");
@@ -1292,7 +1340,12 @@ void wxBitmap::SetSourceSurface(cairo_t* cr, int x, int y, const wxColour* fg, c
         cairo_set_source_surface(cr, bmpData->m_surface, x, y);
         return;
     }
-    wxCHECK_RET(bmpData->m_pixbufNoMask, "no bitmap data");
+
+    // Silently ignore attempts to draw uninitialized bitmap,
+    // because other platforms don't complain about it
+    if (bmpData->m_pixbufNoMask == NULL)
+        return;
+
     if (bmpData->m_bpp == 1)
         SetSourceSurface1(bmpData, cr, x, y, fg, bg);
     else
@@ -1365,6 +1418,52 @@ void wxBitmap::Draw(cairo_t* cr, int x, int y, bool useMask, const wxColour* fg,
     else
         cairo_paint(cr);
 }
+
+wxBitmap wxBitmap::CreateDisabled() const
+{
+    wxBitmap disabled;
+    if (m_refData == NULL)
+        return disabled;
+
+    const wxBitmapRefData* bmpData = M_BMPDATA;
+    wxBitmapRefData* newRef = new wxBitmapRefData(bmpData->m_width, bmpData->m_height, 32);
+    newRef->m_scaleFactor = bmpData->m_scaleFactor;
+    disabled.m_refData = newRef;
+
+    cairo_t* cr = disabled.CairoCreate();
+    cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
+    cairo_set_source_rgba(cr, 0, 0, 0, 0);
+    // clear to transparent
+    cairo_paint(cr);
+    // draw in this bitmap
+    Draw(cr, 0, 0);
+    cairo_set_source_rgba(cr, 0, 0, 0, 0);
+    // create disabled appearance
+    cairo_paint_with_alpha(cr, 0.5);
+    cairo_destroy(cr);
+
+    return disabled;
+}
+#else
+GdkPixbuf* wxBitmap::GetPixbufNoMask() const
+{
+    wxCHECK_MSG(IsOk(), NULL, "invalid bitmap");
+
+    wxBitmapRefData* bmpData = M_BMPDATA;
+    GdkPixbuf* pixbuf = bmpData->m_pixbuf;
+    if (pixbuf)
+        return pixbuf;
+
+    const int w = bmpData->m_width;
+    const int h = bmpData->m_height;
+
+    pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, bmpData->m_alphaRequested, 8, w, h);
+    if ( bmpData->m_pixmap )
+        PixmapToPixbuf(bmpData->m_pixmap, pixbuf, w, h);
+
+    bmpData->m_pixbuf = pixbuf;
+    return pixbuf;
+}
 #endif
 
 GdkPixbuf *wxBitmap::GetPixbuf() const
@@ -1405,21 +1504,31 @@ GdkPixbuf *wxBitmap::GetPixbuf() const
 
     return bmpData->m_pixbufMask;
 #else
-    if (bmpData->m_pixbuf)
-        return bmpData->m_pixbuf;
+    if ( bmpData->m_pixbufMask )
+        return bmpData->m_pixbufMask;
 
     const int w = bmpData->m_width;
     const int h = bmpData->m_height;
+    if ( !bmpData->m_pixbuf )
+        GetPixbufNoMask();
+
     GdkPixmap* mask = NULL;
     if (bmpData->m_mask)
         mask = *bmpData->m_mask;
-    const bool useAlpha = bmpData->m_alphaRequested || mask;
-    bmpData->m_pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, useAlpha, 8, w, h);
-    if (bmpData->m_pixmap)
-        PixmapToPixbuf(bmpData->m_pixmap, bmpData->m_pixbuf, w, h);
-    if (mask)
-        MaskToAlpha(mask, bmpData->m_pixbuf, w, h);
-    return bmpData->m_pixbuf;
+
+    if ( !mask )
+        return bmpData->m_pixbuf;
+
+    bmpData->m_pixbufMask = gdk_pixbuf_new(GDK_COLORSPACE_RGB, true, 8, w, h);
+    guchar* dst = gdk_pixbuf_get_pixels(bmpData->m_pixbufMask);
+    guchar* src = gdk_pixbuf_get_pixels(bmpData->m_pixbuf);
+    const int dstStride = gdk_pixbuf_get_rowstride(bmpData->m_pixbufMask);
+    const int srcStride = gdk_pixbuf_get_rowstride(bmpData->m_pixbuf);
+    CopyImageData(dst, 4, dstStride,
+                  src, gdk_pixbuf_get_n_channels(bmpData->m_pixbuf), srcStride,
+                  w, h);
+    MaskToAlpha(mask, bmpData->m_pixbufMask, w, h);
+    return bmpData->m_pixbufMask;
 #endif
 }
 
@@ -1471,11 +1580,17 @@ void *wxBitmap::GetRawData(wxPixelDataBase& data, int bpp)
         }
     }
 #else
-    GdkPixbuf *pixbuf = GetPixbuf();
+    GdkPixbuf *pixbuf = GetPixbufNoMask();
 
     // Pixmap will get out of date when our pixbuf is accessed directly, so
     // ensure we don't keep the old data in it.
     PurgeOtherRepresentations(Pixbuf);
+    // Pixbuf with masked data will get out of date too
+    if ( M_BMPDATA->m_pixbufMask )
+    {
+        g_object_unref(M_BMPDATA->m_pixbufMask);
+        M_BMPDATA->m_pixbufMask = NULL;
+    }
 
     const bool hasAlpha = HasAlpha();
 
