@@ -11,32 +11,17 @@
 // For compilers that support precompilation, includes "wx/wx.h".
 #include "wx/wxprec.h"
 
-#ifdef __BORLANDC__
-    #pragma hdrstop
-#endif
 
 #if wxUSE_PROPGRID
 
 #ifndef WX_PRECOMP
-    #include "wx/defs.h"
-    #include "wx/object.h"
-    #include "wx/hash.h"
-    #include "wx/string.h"
+    #include "wx/bitmap.h"
+    #include "wx/colour.h"
+    #include "wx/font.h"
     #include "wx/log.h"
-    #include "wx/event.h"
-    #include "wx/window.h"
-    #include "wx/panel.h"
-    #include "wx/dc.h"
-    #include "wx/dcmemory.h"
-    #include "wx/button.h"
-    #include "wx/pen.h"
-    #include "wx/brush.h"
-    #include "wx/settings.h"
-    #include "wx/sizer.h"
-    #include "wx/intl.h"
+    #include "wx/validate.h"
 #endif
 
-#include "wx/propgrid/property.h"
 #include "wx/propgrid/propgrid.h"
 
 // ----------------------------------------------------------------------------
@@ -54,19 +39,13 @@ IMPLEMENT_VARIANT_OBJECT_EXPORTED(wxFont, WXDLLIMPEXP_PROPGRID)
 
 wxPGProperty* wxPGPropArgCls::GetPtr( wxPropertyGridInterface* iface ) const
 {
-    if ( m_flags == IsProperty )
+    if ( m_isProperty )
     {
-        wxASSERT_MSG( m_ptr.property, wxS("invalid property ptr") );
-        return m_ptr.property;
+        wxASSERT_MSG( m_property, "invalid property ptr" );
+        return m_property;
     }
-    else if ( m_flags & IsWxString )
-        return iface->GetPropertyByNameA(*m_ptr.stringName);
-    else if ( m_flags & IsCharPtr )
-        return iface->GetPropertyByNameA(m_ptr.charName);
-    else if ( m_flags & IsWCharPtr )
-        return iface->GetPropertyByNameA(m_ptr.wcharName);
 
-    return NULL;
+    return iface->GetPropertyByNameA(m_name);
 }
 
 // -----------------------------------------------------------------------
@@ -103,8 +82,7 @@ wxPGProperty* wxPropertyGridInterface::Append( wxPGProperty* property )
 wxPGProperty* wxPropertyGridInterface::AppendIn( wxPGPropArg id, wxPGProperty* newproperty )
 {
     wxPG_PROP_ARG_CALL_PROLOG_RETVAL(wxNullProperty)
-    wxPGProperty* pwc = (wxPGProperty*) p;
-    wxPGProperty* retp = m_pState->DoInsert(pwc, pwc->GetChildCount(), newproperty);
+    wxPGProperty* retp = m_pState->DoInsert(p, p->GetChildCount(), newproperty);
     return retp;
 }
 
@@ -123,7 +101,7 @@ wxPGProperty* wxPropertyGridInterface::Insert( wxPGPropArg id, wxPGProperty* pro
 wxPGProperty* wxPropertyGridInterface::Insert( wxPGPropArg id, int index, wxPGProperty* newproperty )
 {
     wxPG_PROP_ARG_CALL_PROLOG_RETVAL(wxNullProperty)
-    wxPGProperty* retp = m_pState->DoInsert((wxPGProperty*)p,index,newproperty);
+    wxPGProperty* retp = m_pState->DoInsert(p,index,newproperty);
     RefreshGrid();
     return retp;
 }
@@ -323,7 +301,7 @@ bool wxPropertyGridInterface::ExpandAll( bool doExpand )
 
     for ( it = GetVIterator( wxPG_ITERATE_ALL ); !it.AtEnd(); it.Next() )
     {
-        wxPGProperty* p = (wxPGProperty*) it.GetProperty();
+        wxPGProperty* p = it.GetProperty();
         if ( p->GetChildCount() )
         {
             if ( doExpand )
@@ -399,7 +377,7 @@ void wxPGTypeOperationFailed( const wxPGProperty* p,
 {
     wxASSERT( p != NULL );
     wxLogError( _("Type operation \"%s\" failed: Property labeled \"%s\" is of type \"%s\", NOT \"%s\"."),
-        op.c_str(), p->GetLabel().c_str(), p->GetValue().GetType().c_str(), typestr.c_str() );
+        op, p->GetLabel(), p->GetValue().GetType(), typestr );
 }
 
 // -----------------------------------------------------------------------
@@ -434,7 +412,7 @@ void wxPropertyGridInterface::SetValidationFailureBehavior( int vfbFlags )
 wxPGProperty* wxPropertyGridInterface::GetPropertyByNameA( const wxString& name ) const
 {
     wxPGProperty* p = GetPropertyByName(name);
-    wxASSERT_MSG(p,wxString::Format(wxS("no property with name '%s'"),name.c_str()));
+    wxASSERT_MSG(p,wxString::Format(wxS("no property with name '%s'"),name));
     return p;
 }
 
@@ -452,13 +430,7 @@ void wxPropertyGridInterface::DoSetPropertyAttribute( wxPGPropArg id, const wxSt
 {
     wxPG_PROP_ARG_CALL_PROLOG()
 
-    p->SetAttribute( name, value );
-    // If property is attached to the property grid
-    // then refresh the view.
-    if( p->GetParentState() )
-    {
-        RefreshProperty( p );
-    }
+    p->SetAttribute( name, value ); // property is also refreshed here
 
     if ( argFlags & wxPG_RECURSE )
     {
@@ -499,17 +471,17 @@ void wxPropertyGridInterface::GetPropertiesWithFlag( wxArrayPGProperty* targetAr
           !it.AtEnd();
           it.Next() )
     {
-        const wxPGProperty* property = it.GetProperty();
+        wxPGProperty* property = it.GetProperty();
 
         if ( !inverse )
         {
             if ( property->HasFlagsExact(flags) )
-                targetArr->push_back((wxPGProperty*)property);
+                targetArr->push_back(property);
         }
         else
         {
             if ( !property->HasFlagsExact(flags) )
-                targetArr->push_back((wxPGProperty*)property);
+                targetArr->push_back(property);
         }
     }
 }
@@ -673,20 +645,18 @@ bool wxPropertyGridInterface::SetPropertyMaxLength( wxPGPropArg id, int maxLen )
 {
     wxPG_PROP_ARG_CALL_PROLOG_RETVAL(false)
 
-    wxPropertyGrid* pg = m_pState->GetGrid();
+    if ( !p->SetMaxLength(maxLen) )
+        return false;
 
-    p->m_maxLen = (short) maxLen;
+    wxPropertyGrid* pg = m_pState->GetGrid();
 
     // Adjust control if selected currently
     if ( pg == p->GetGrid() && p == m_pState->GetSelection() )
     {
         wxWindow* wnd = pg->GetEditorControl();
-        wxTextCtrl* tc = wxDynamicCast(wnd,wxTextCtrl);
-        if ( tc )
-            tc->SetMaxLength( maxLen );
-        else
-        // Not a text ctrl
-            return false;
+        wxTextCtrl* tc = wxDynamicCast(wnd, wxTextCtrl);
+        wxCHECK_MSG(tc, false, "Text ctrl is expected here");
+        tc->SetMaxLength(maxLen);
     }
 
     return true;
@@ -763,7 +733,7 @@ void wxPropertyGridInterface::SetPropertyColoursToDefault(wxPGPropArg id, int fl
 void wxPropertyGridInterface::SetPropertyCell( wxPGPropArg id,
                                                int column,
                                                const wxString& text,
-                                               const wxBitmap& bitmap,
+                                               const wxBitmapBundle& bitmap,
                                                const wxColour& fgCol,
                                                const wxColour& bgCol )
 {
@@ -783,20 +753,13 @@ void wxPropertyGridInterface::SetPropertyCell( wxPGPropArg id,
 // -----------------------------------------------------------------------
 // GetPropertyValueAsXXX methods
 
-#define IMPLEMENT_GET_VALUE(TRET,PGTypeName,BIGNAME,DEFRETVAL) \
-TRET wxPropertyGridInterface::GetPropertyValueAs##BIGNAME( wxPGPropArg id ) const \
-{ \
-    wxPG_PROP_ARG_CALL_PROLOG_RETVAL(DEFRETVAL) \
-    wxVariant value = p->GetValue(); \
-    if ( !value.IsType(PGTypeName) ) \
-    { \
-        wxPGGetFailed(p, PGTypeName); \
-        return (TRET)DEFRETVAL; \
-    } \
-    return (TRET)value.Get##BIGNAME(); \
+wxVariant wxPropertyGridInterface::GetPropertyValue(wxPGPropArg id)
+{
+    wxPG_PROP_ARG_CALL_PROLOG_RETVAL(wxVariant())
+    return p->GetValue();
 }
 
-// String is different than others.
+// String is different from others.
 wxString wxPropertyGridInterface::GetPropertyValueAsString( wxPGPropArg id ) const
 {
     wxPG_PROP_ARG_CALL_PROLOG_RETVAL(wxEmptyString)
@@ -819,8 +782,62 @@ bool wxPropertyGridInterface::GetPropertyValueAsBool( wxPGPropArg id ) const
     return false;
 }
 
-IMPLEMENT_GET_VALUE(long,wxPG_VARIANT_TYPE_LONG,Long,0)
-IMPLEMENT_GET_VALUE(double,wxPG_VARIANT_TYPE_DOUBLE,Double,0.0)
+#define wxPG_PROP_ID_GETPROPVAL_CALL_PROLOG_RETVAL(PGTypeName, DEFVAL) \
+    wxPG_PROP_ARG_CALL_PROLOG_RETVAL(DEFVAL) \
+    wxVariant value = p->GetValue(); \
+    if ( !value.IsType(PGTypeName) ) \
+    { \
+        wxPGGetFailed(p, PGTypeName); \
+        return DEFVAL; \
+    }
+
+long wxPropertyGridInterface::GetPropertyValueAsLong(wxPGPropArg id) const
+{
+    wxPG_PROP_ID_GETPROPVAL_CALL_PROLOG_RETVAL(wxPG_VARIANT_TYPE_LONG, 0L)
+    return value.GetLong();
+}
+
+double wxPropertyGridInterface::GetPropertyValueAsDouble(wxPGPropArg id) const
+{
+    wxPG_PROP_ID_GETPROPVAL_CALL_PROLOG_RETVAL(wxPG_VARIANT_TYPE_DOUBLE, 0.0)
+    return value.GetDouble();
+}
+
+wxArrayString wxPropertyGridInterface::GetPropertyValueAsArrayString(wxPGPropArg id) const
+{
+    wxPG_PROP_ID_GETPROPVAL_CALL_PROLOG_RETVAL(wxPG_VARIANT_TYPE_ARRSTRING, wxArrayString())
+    return value.GetArrayString();
+}
+
+#if defined(wxLongLong_t) && wxUSE_LONGLONG
+wxLongLong_t wxPropertyGridInterface::GetPropertyValueAsLongLong(wxPGPropArg id) const
+{
+    wxPG_PROP_ARG_CALL_PROLOG_RETVAL(0)
+    return p->GetValue().GetLongLong().GetValue();
+}
+
+wxULongLong_t wxPropertyGridInterface::GetPropertyValueAsULongLong(wxPGPropArg id) const
+{
+    wxPG_PROP_ARG_CALL_PROLOG_RETVAL(0)
+    return p->GetValue().GetULongLong().GetValue();
+}
+#endif
+
+wxArrayInt wxPropertyGridInterface::GetPropertyValueAsArrayInt(wxPGPropArg id) const
+{
+    wxPG_PROP_ID_GETPROPVAL_CALL_PROLOG_RETVAL(wxArrayInt_VariantType, wxArrayInt())
+    wxArrayInt arr;
+    arr << value;
+    return arr;
+}
+
+#if wxUSE_DATETIME
+wxDateTime wxPropertyGridInterface::GetPropertyValueAsDateTime(wxPGPropArg id) const
+{
+    wxPG_PROP_ID_GETPROPVAL_CALL_PROLOG_RETVAL(wxPG_VARIANT_TYPE_DATETIME, wxDateTime())
+    return value.GetDateTime();
+}
+#endif
 
 bool wxPropertyGridInterface::IsPropertyExpanded( wxPGPropArg id ) const
 {
@@ -896,15 +913,16 @@ wxPGVIterator wxPropertyGridInterface::GetVIterator( int flags ) const
 static wxString EscapeDelimiters(const wxString& s)
 {
     wxString result;
-    result.Alloc(s.length());
-    const wxChar* ch = s.c_str();
-    while (*ch)
+    result.reserve(s.length());
+
+    for (wxString::const_iterator it = s.begin(); it != s.end(); ++it)
     {
-        if (*ch == wxT(';') || *ch == wxT('|') || *ch == wxT(','))
-            result += wxT('\\');
-        result += *ch;
-        ++ch;
+        wxStringCharType ch = *it;
+        if ( ch == wxS(';') || ch == wxS('|') || ch == wxS(',') )
+            result += wxS('\\');
+        result += ch;
     }
+
     return result;
 }
 
@@ -914,22 +932,19 @@ wxString wxPropertyGridInterface::SaveEditableState( int includedStates ) const
 
     //
     // Save state on page basis
-    unsigned int pageIndex = 0;
-    wxArrayPtrVoid pageStates;
-
-    for (;;)
+    wxVector<wxPropertyGridPageState*> pageStates;
+    for (int pageIndex = 0; ; pageIndex++)
     {
         wxPropertyGridPageState* page = GetPageState(pageIndex);
         if ( !page ) break;
 
-        pageStates.Add(page);
-
-        pageIndex++;
+        pageStates.push_back(page);
     }
 
-    for ( pageIndex=0; pageIndex < pageStates.size(); pageIndex++ )
+    for (wxVector<wxPropertyGridPageState*>::const_iterator it_ps = pageStates.begin();
+         it_ps != pageStates.end(); ++it_ps)
     {
-        wxPropertyGridPageState* pageState = (wxPropertyGridPageState*) pageStates[pageIndex];
+        wxPropertyGridPageState* pageState = *it_ps;
 
         if ( includedStates & SelectionState )
         {
@@ -942,7 +957,6 @@ wxString wxPropertyGridInterface::SaveEditableState( int includedStates ) const
         }
         if ( includedStates & ExpandedState )
         {
-            wxArrayPGProperty ptrs;
             wxPropertyGridConstIterator it =
                 wxPropertyGridConstIterator( pageState,
                                              wxPG_ITERATE_ALL_PARENTS_RECURSIVELY|wxPG_ITERATE_HIDDEN,
@@ -1088,7 +1102,7 @@ bool wxPropertyGridInterface::RestoreEditableState( const wxString& src, int res
                 {
                     if ( restoreStates & SplitterPosState )
                     {
-                        for ( size_t n=1; n<values.size(); n++ )
+                        for ( size_t n=0; n<values.size(); n++ )
                         {
                             long pos = 0;
                             values[n].ToLong(&pos);
@@ -1184,7 +1198,12 @@ bool wxPropertyGridInterface::RestoreEditableState( const wxString& src, int res
 
     if ( selectedPage != -1 )
     {
+        wxPropertyGridPageState* curPageState = GetPageState(-1);
         DoSelectPage(selectedPage);
+        if ( GetPageState(-1) != curPageState )
+        {
+            pg->SendEvent(wxEVT_PG_SELECTED, pg->GetSelectedProperty());
+        }
     }
 
     if ( vx >= 0 )
