@@ -2,16 +2,12 @@
 // Name:        tests/filetype/filetype.cpp
 // Purpose:     Test wxGetFileKind and wxStreamBase::IsSeekable
 // Author:      Mike Wetherell
-// RCS-ID:      $Id$
 // Copyright:   (c) 2005 Mike Wetherell
 // Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
 
 #include "testprec.h"
 
-#ifdef __BORLANDC__
-    #pragma hdrstop
-#endif
 
 // for all others, include the necessary headers
 #ifndef WX_PRECOMP
@@ -38,6 +34,8 @@
     #define fileno _fileno
 #endif
 
+#include "testfile.h"
+
 ///////////////////////////////////////////////////////////////////////////////
 // The test case
 
@@ -72,7 +70,7 @@ class FileKindTestCase : public CppUnit::TestCase
 };
 
 // test a wxFFile and wxFFileInput/OutputStreams of a known type
-// 
+//
 void FileKindTestCase::TestFILE(wxFFile& file, bool expected)
 {
     CPPUNIT_ASSERT(file.IsOpened());
@@ -101,23 +99,17 @@ void FileKindTestCase::TestFd(wxFile& file, bool expected)
     CPPUNIT_ASSERT(outStream.IsSeekable() == expected);
 }
 
-struct TempFile
-{
-    ~TempFile() { if (!m_name.IsEmpty()) wxRemoveFile(m_name); }
-    wxString m_name;
-};
-
 // test with an ordinary file
 //
 void FileKindTestCase::File()
 {
     TempFile tmp; // put first
     wxFile file;
-    tmp.m_name = wxFileName::CreateTempFileName(wxT("wxft"), &file);
+    tmp.Assign(wxFileName::CreateTempFileName(wxT("wxft"), &file));
     TestFd(file, true);
     file.Close();
 
-    wxFFile ffile(tmp.m_name);
+    wxFFile ffile(tmp.GetName());
     TestFILE(ffile, true);
 }
 
@@ -127,11 +119,13 @@ void FileKindTestCase::File()
 void FileKindTestCase::Pipe()
 {
     int afd[2];
+    int rc;
 #ifdef __UNIX__
-    pipe(afd);
+    rc = pipe(afd);
 #else
-    _pipe(afd, 256, O_BINARY);
+    rc = _pipe(afd, 256, O_BINARY);
 #endif
+    CPPUNIT_ASSERT_EQUAL_MESSAGE("Failed to create pipe", 0, rc);
 
     wxFile file0(afd[0]);
     wxFile file1(afd[1]);
@@ -194,7 +188,7 @@ void FileKindTestCase::MemoryStream()
 }
 
 // Stdin will usually be a terminal, if so then test it
-// 
+//
 void FileKindTestCase::Stdin()
 {
     if (isatty(0))
@@ -206,7 +200,7 @@ void FileKindTestCase::Stdin()
 // register in the unnamed registry so that these tests are run by default
 CPPUNIT_TEST_SUITE_REGISTRATION(FileKindTestCase);
 
-// also include in it's own registry so that these tests can be run alone
+// also include in its own registry so that these tests can be run alone
 CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(FileKindTestCase, "FileKindTestCase");
 
 #endif // wxUSE_STREAMS
